@@ -750,12 +750,12 @@ void tell_bottree(int idx, int showver)
             more = 1;
             this = last[lev];
           }
-          dprintf(idx, "------------------------------------------------\n");
         }
       }
     }
   }
   /* Hop information: (9d) */
+  dprintf(idx, "------------------------------------------------\n");
   dprintf(idx, "Average hops: %3.1f, total bots: %d\n",
           ((float) tothops) / ((float) tands), tands + 1);
 }
@@ -902,7 +902,7 @@ int users_in_subtree(tand_t *bot)
  */
 int botunlink(int idx, char *nick, char *reason, char *from)
 {
-  char s[20];
+  char s[1024];
   int i;
   int bots, users;
   tand_t *bot;
@@ -931,8 +931,6 @@ int botunlink(int idx, char *nick, char *reason, char *from)
         if (nick[0] != '*')
           return 1;
       } else if (dcc[i].type == &DCC_BOT) {
-        char s[1024];
-
         if (idx >= 0)
           dprintf(idx, "%s %s.\n", BOT_BREAKLINK, dcc[i].nick);
         else if ((idx == -3) && (b_status(i) & STAT_SHARE) && !share_unlinks)
@@ -941,16 +939,16 @@ int botunlink(int idx, char *nick, char *reason, char *from)
         bots = bots_in_subtree(bot);
         users = users_in_subtree(bot);
         if (reason && reason[0]) {
-          simple_sprintf(s, "%s %s (%s (%s)) (lost %d bot%s and %d user%s)",
-                         BOT_UNLINKEDFROM, dcc[i].nick, reason, from, bots,
-                         (bots != 1) ? "s" : "", users, (users != 1) ?
-                         "s" : "");
+          snprintf(s, sizeof s, "%s %s (%s (%s)) (lost %d bot%s and %d user%s)",
+                   BOT_UNLINKEDFROM, dcc[i].nick, reason, from, bots,
+                   (bots != 1) ? "s" : "", users, (users != 1) ?
+                   "s" : "");
           dprintf(i, "bye %s\n", reason);
         } else {
-          simple_sprintf(s, "%s %s (%s) (lost %d bot%s and %d user%s)",
-                         BOT_UNLINKEDFROM, dcc[i].nick, from, bots,
-                         (bots != 1) ? "s" : "", users,
-                         (users != 1) ? "s" : "");
+          snprintf(s, sizeof s, "%s %s (%s) (lost %d bot%s and %d user%s)",
+                   BOT_UNLINKEDFROM, dcc[i].nick, from, bots,
+                   (bots != 1) ? "s" : "", users,
+                   (users != 1) ? "s" : "");
           dprintf(i, "bye No reason\n");
         }
         putlog(LOG_BOTS, "*", "%s.", s);
@@ -997,8 +995,7 @@ int botunlink(int idx, char *nick, char *reason, char *from)
         check_tcl_chpt(party[i].bot, party[i].nick, party[i].sock,
                        party[i].chan);
     }
-    strcpy(s, "killassoc &");
-    Tcl_Eval(interp, s);
+    Tcl_Eval(interp, "killassoc &");
   }
   return 0;
 }
@@ -1138,7 +1135,7 @@ static void failed_tandem_relay(int idx)
         (dcc[i].u.relay->sock == dcc[idx].sock))
       uidx = i;
   if (uidx < 0) {
-    putlog(LOG_MISC, "*", "%s  %d -> %d", BOT_CANTFINDRELAYUSER,
+    putlog(LOG_MISC, "*", "%s  %ld -> %d", BOT_CANTFINDRELAYUSER,
            dcc[idx].sock, dcc[idx].u.relay->sock);
     killsock(dcc[idx].sock);
     lostdcc(idx);
@@ -1261,7 +1258,7 @@ static void tandem_relay_resolve_failure(int idx)
       break;
     }
   if (uidx < 0) {
-    putlog(LOG_MISC, "*", "%s  %d -> %d", BOT_CANTFINDRELAYUSER,
+    putlog(LOG_MISC, "*", "%s  %ld -> %d", BOT_CANTFINDRELAYUSER,
            dcc[idx].sock, dcc[idx].u.relay->sock);
     killsock(dcc[idx].sock);
     lostdcc(idx);
@@ -1334,7 +1331,7 @@ static void pre_relay(int idx, char *buf, int i)
       }
   }
   if (tidx < 0) {
-    putlog(LOG_MISC, "*", "%s  %d -> %d", BOT_CANTFINDRELAYUSER,
+    putlog(LOG_MISC, "*", "%s  %ld -> %d", BOT_CANTFINDRELAYUSER,
            dcc[idx].sock, dcc[idx].u.relay->sock);
     killsock(dcc[idx].sock);
     lostdcc(idx);
@@ -1380,7 +1377,7 @@ static void failed_pre_relay(int idx)
       }
   }
   if (tidx < 0) {
-    putlog(LOG_MISC, "*", "%s  %d -> %d", BOT_CANTFINDRELAYUSER,
+    putlog(LOG_MISC, "*", "%s  %ld -> %d", BOT_CANTFINDRELAYUSER,
            dcc[idx].sock, dcc[idx].u.relay->sock);
     killsock(dcc[idx].sock);
     lostdcc(idx);
@@ -1414,7 +1411,7 @@ static void cont_tandem_relay(int idx, char *buf, int i)
         (dcc[i].u.relay->sock == dcc[idx].sock))
       uidx = i;
   if (uidx < 0) {
-    putlog(LOG_MISC, "*", "%s  %d -> %d", BOT_CANTFINDRELAYUSER,
+    putlog(LOG_MISC, "*", "%s  %ld -> %d", BOT_CANTFINDRELAYUSER,
            dcc[i].sock, dcc[i].u.relay->sock);
     killsock(dcc[i].sock);
     lostdcc(i);
@@ -1632,6 +1629,7 @@ struct dcc_table DCC_RELAY = {
   display_relay,
   expmem_relay,
   kill_relay,
+  NULL,
   NULL
 };
 
@@ -1655,7 +1653,8 @@ struct dcc_table DCC_RELAYING = {
   display_relaying,
   expmem_relay,
   kill_relay,
-  out_relay
+  out_relay,
+  NULL
 };
 
 struct dcc_table DCC_FORK_RELAY = {
@@ -1668,6 +1667,7 @@ struct dcc_table DCC_FORK_RELAY = {
   display_tandem_relay,
   expmem_relay,
   kill_relay,
+  NULL,
   NULL
 };
 
@@ -1681,6 +1681,7 @@ struct dcc_table DCC_PRE_RELAY = {
   display_pre_relay,
   expmem_relay,
   kill_relay,
+  NULL,
   NULL
 };
 
