@@ -237,14 +237,18 @@ static void bot_version(int idx, char *par)
 #ifndef NO_OLD_BOTNET
   }
 #endif
-  strlcpy(dcc[idx].u.bot->version, par, 120);
+  strlcpy(dcc[idx].u.bot->version, par, sizeof dcc[idx].u.bot->version);
   putlog(LOG_BOTS, "*", DCC_LINKED, dcc[idx].nick);
   botnet_send_nlinked(idx, dcc[idx].nick, botnetnick, '!',
                       dcc[idx].u.bot->numver);
   touch_laston(dcc[idx].user, "linked", now);
   dump_links(idx);
   dcc[idx].type = &DCC_BOT;
+#ifdef TLS
   addbot(dcc[idx].nick, dcc[idx].nick, botnetnick, '-', dcc[idx].u.bot->numver, dcc[idx].ssl);
+#else
+  addbot(dcc[idx].nick, dcc[idx].nick, botnetnick, '-', dcc[idx].u.bot->numver, 0);
+#endif
   check_tcl_link(dcc[idx].nick, botnetnick);
   egg_snprintf(x, sizeof x, "v %d", dcc[idx].u.bot->numver);
   bot_share(idx, x);
@@ -466,7 +470,8 @@ struct dcc_table DCC_BOT_NEW = {
   display_dcc_bot_new,
   expmem_dcc_bot_,
   free_dcc_bot_,
-  out_dcc_bot
+  out_dcc_bot,
+  NULL
 };
 
 /* Hash function for tandem bot commands */
@@ -520,7 +525,7 @@ static void eof_dcc_bot(int idx)
 
 static void display_dcc_bot(int idx, char *buf)
 {
-  int i = simple_sprintf(buf, "bot   flags: ");
+  int i = sprintf(buf, "bot   flags: ");
 
   buf[i++] = b_status(idx) & STAT_PINGED ? 'P' : 'p';
   buf[i++] = b_status(idx) & STAT_SHARE ? 'U' : 'u';
@@ -550,7 +555,8 @@ struct dcc_table DCC_BOT = {
   display_dcc_bot,
   expmem_dcc_bot_,
   free_dcc_bot_,
-  out_dcc_bot
+  out_dcc_bot,
+  NULL
 };
 
 struct dcc_table DCC_FORK_BOT = {
@@ -563,7 +569,8 @@ struct dcc_table DCC_FORK_BOT = {
   display_dcc_fork_bot,
   expmem_dcc_bot_,
   free_dcc_bot_,
-  out_dcc_bot
+  out_dcc_bot,
+  NULL
 };
 
 /* This function generates a digest by combining a challenge consisting
@@ -586,8 +593,8 @@ static int dcc_bot_check_digest(int idx, char *remote_digest)
 
   MD5_Init(&md5context);
 
-  egg_snprintf(digest_string, 33, "<%lx%x@", (long) getpid(),
-               (unsigned int) dcc[idx].timeval);
+  egg_snprintf(digest_string, 33, "<%lx%lx@", (long) getpid(),
+               (unsigned long) dcc[idx].timeval);
   MD5_Update(&md5context, (unsigned char *) digest_string,
              strlen(digest_string));
   MD5_Update(&md5context, (unsigned char *) botnetnick, strlen(botnetnick));
@@ -715,7 +722,7 @@ static void dcc_chat_pass(int idx, char *buf, int atr)
       dprintf(idx, "badpass\n");
       dcc[idx].type = old;
     } else
-      dprintf(idx, DCC_HOUSTON);
+      dprintf(idx, "%s", DCC_HOUSTON);
     putlog(LOG_MISC, "*", DCC_BADLOGIN, dcc[idx].nick,
            dcc[idx].host, dcc[idx].port);
     if (dcc[idx].u.chat->away) {        /* su from a dumb user */
@@ -955,7 +962,8 @@ struct dcc_table DCC_CHAT_PASS = {
   display_dcc_chat_pass,
   expmem_dcc_general,
   kill_dcc_general,
-  out_dcc_general
+  out_dcc_general,
+  NULL
 };
 
 /* Make sure ANSI code is just for color-changing */
@@ -1155,7 +1163,7 @@ static void dcc_chat(int idx, char *buf, int i)
 
 static void display_dcc_chat(int idx, char *buf)
 {
-  int i = simple_sprintf(buf, "chat  flags: ");
+  int i = sprintf(buf, "chat  flags: ");
 
   buf[i++] = dcc[idx].status & STAT_CHAT ? 'C' : 'c';
   buf[i++] = dcc[idx].status & STAT_PARTY ? 'P' : 'p';
@@ -1176,7 +1184,8 @@ struct dcc_table DCC_CHAT = {
   display_dcc_chat,
   expmem_dcc_general,
   kill_dcc_general,
-  out_dcc_general
+  out_dcc_general,
+  NULL
 };
 
 static int lasttelnets;
@@ -1365,8 +1374,11 @@ static void dcc_telnet_hostresolved(int i)
     if (dcc[j].sock >= 0) {
       sockname_t name;
       name.addrlen = sizeof(name.addr);
-      getsockname(dcc[i].sock, &name.addr.sa, &name.addrlen);
-      bind(dcc[j].sock, &name.addr.sa, name.addrlen);
+      if (getsockname(dcc[i].sock, &name.addr.sa, &name.addrlen) < 0)
+        debug2("dcc: dcc_telnet_hostresolved(): getsockname() socket %ld error %s", dcc[i].sock, strerror(errno));
+      setsnport(name, 0);
+      if (bind(dcc[j].sock, &name.addr.sa, name.addrlen) < 0)
+        debug2("dcc: dcc_telnet_hostresolved(): bind() socket %ld error %s", dcc[j].sock, strerror(errno));
       setsnport(dcc[j].sockname, 113);
       if (connect(dcc[j].sock, &dcc[j].sockname.addr.sa,
           dcc[j].sockname.addrlen) < 0 && (errno != EINPROGRESS)) {
@@ -1420,6 +1432,7 @@ struct dcc_table DCC_TELNET = {
   display_telnet,
   NULL,
   NULL,
+  NULL,
   NULL
 };
 
@@ -1492,7 +1505,8 @@ struct dcc_table DCC_DUPWAIT = {
   display_dupwait,
   expmem_dupwait,
   kill_dupwait,
-  out_dcc_bot
+  out_dcc_bot,
+  NULL
 };
 
 /* This function is called if a bot gets removed from the list. It checks
@@ -1586,7 +1600,7 @@ static void dcc_telnet_id(int idx, char *buf, int atr)
     dcc[idx].timeval = now;
     dprintf(idx, "\n");
     dprintf(idx, IRC_TELNET, botnetnick);
-    dprintf(idx, IRC_TELNET1);
+    dprintf(idx, "%s", IRC_TELNET1);
     dprintf(idx, "\nEnter the handle you would like to use.\n");
     return;
   }
@@ -1763,7 +1777,7 @@ static void dcc_telnet_pass(int idx, int atr)
      */
     putlog(LOG_BOTS, "*", "Challenging %s...", dcc[idx].nick);
     /* Prefix with \n in case of newline-less ending stealth_prompt */
-    dprintf(idx, "\npassreq <%x%x@%s>\n", getpid(), dcc[idx].timeval, botnetnick);
+    dprintf(idx, "\npassreq <%x%lx@%s>\n", getpid(), (unsigned long)dcc[idx].timeval, botnetnick);
     dcc[idx].type = old;
   } else {
     /* NOTE: The MD5 digest used above to prevent cleartext passwords being
@@ -1818,7 +1832,8 @@ struct dcc_table DCC_TELNET_ID = {
   display_dcc_telnet_id,
   expmem_dcc_general,
   kill_dcc_general,
-  out_dcc_general
+  out_dcc_general,
+  NULL
 };
 
 static void dcc_telnet_new(int idx, char *buf, int x)
@@ -1877,7 +1892,7 @@ static void dcc_telnet_new(int idx, char *buf, int x)
     dcc[idx].type = &DCC_TELNET_PW;
     if (make_userfile) {
       dprintf(idx, "\nYOU ARE THE MASTER/OWNER ON THIS BOT NOW\n");
-      dprintf(idx, IRC_LIMBO);
+      dprintf(idx, "%s", IRC_LIMBO);
       putlog(LOG_MISC, "*", DCC_INSTCOMPL, buf);
       make_userfile = 0;
       write_userfile(-1);
@@ -1979,7 +1994,8 @@ struct dcc_table DCC_TELNET_NEW = {
   display_dcc_telnet_new,
   expmem_dcc_general,
   kill_dcc_general,
-  out_dcc_general
+  out_dcc_general,
+  NULL
 };
 
 struct dcc_table DCC_TELNET_PW = {
@@ -1992,7 +2008,8 @@ struct dcc_table DCC_TELNET_PW = {
   display_dcc_telnet_pw,
   expmem_dcc_general,
   kill_dcc_general,
-  out_dcc_general
+  out_dcc_general,
+  NULL
 };
 
 static int call_tcl_func(char *name, int idx, char *args)
@@ -2123,7 +2140,8 @@ struct dcc_table DCC_SCRIPT = {
   display_dcc_script,
   expmem_dcc_script,
   kill_dcc_script,
-  out_dcc_script
+  out_dcc_script,
+  NULL
 };
 
 static void dcc_socket(int idx, char *buf, int len)
@@ -2151,6 +2169,7 @@ struct dcc_table DCC_SOCKET = {
   display_dcc_socket,
   NULL,
   NULL,
+  NULL,
   NULL
 };
 
@@ -2169,6 +2188,7 @@ struct dcc_table DCC_LOST = {
   display_dcc_lost,
   NULL,
   NULL,
+  NULL,
   NULL
 };
 
@@ -2213,27 +2233,29 @@ struct dcc_table DCC_IDENTWAIT = {
   display_dcc_identwait,
   NULL,
   NULL,
+  NULL,
   NULL
 };
 
 void dcc_ident(int idx, char *buf, int len)
 {
-  char response[512], uid[512], buf1[UHOSTLEN];
+  char response[513], uid[513], buf1[sizeof uid + UHOSTLEN];
   int i;
 
   *response = *uid = '\0';
-  sscanf(buf, "%*[^:]:%[^:]:%*[^:]:%[^\n]\n", response, uid);
+  sscanf(buf, "%*[^:]:%512[^:]:%*[^:]:%512[^\n]\n", response, uid);
   rmspace(response);
-  if (response[0] != 'U') {
+  rmspace(uid);
+  uid[sizeof uid - 1] = '\0';
+  if (strncasecmp(response, "USERID", 6) || strchr(uid, '@')) {
+    debug0("dcc: invalid ident string received, ignoring...");
     dcc[idx].timeval = now;
     return;
   }
-  rmspace(uid);
-  uid[20] = 0;                  /* 20 character ident max */
   for (i = 0; i < dcc_total; i++)
     if ((dcc[i].type == &DCC_IDENTWAIT) &&
         (dcc[i].sock == dcc[idx].u.ident_sock)) {
-      simple_sprintf(buf1, "%s@%s", uid, dcc[idx].host);
+      snprintf(buf1, sizeof buf1, "%s@%s", uid, dcc[idx].host);
       dcc_telnet_got_ident(i, buf1);
     }
   dcc[idx].u.other = 0;
@@ -2243,14 +2265,14 @@ void dcc_ident(int idx, char *buf, int len)
 
 void eof_timeout_dcc_ident(int idx, const char *s)
 {
-  char buf[UHOSTLEN];
+  char buf[7 + UHOSTLEN];
   int i;
 
   for (i = 0; i < dcc_total; i++)
     if ((dcc[i].type == &DCC_IDENTWAIT) &&
         (dcc[i].sock == dcc[idx].u.ident_sock)) {
-      putlog(LOG_MISC, "*", s);
-      simple_sprintf(buf, "telnet@%s", dcc[idx].host);
+      putlog(LOG_MISC, "*", "%s", s);
+      snprintf(buf, sizeof buf, "telnet@%s", dcc[idx].host);
       dcc_telnet_got_ident(i, buf);
     }
   killsock(dcc[idx].sock);
@@ -2283,6 +2305,7 @@ struct dcc_table DCC_IDENT = {
   display_dcc_ident,
   NULL,
   NULL,
+  NULL,
   NULL
 };
 
@@ -2297,7 +2320,7 @@ static void dcc_telnet_got_ident(int i, char *host)
       break;
   dcc[i].u.other = 0;
   if (dcc_total == idx) {
-    putlog(LOG_MISC, "*", DCC_LOSTIDENT);
+    putlog(LOG_MISC, "*", "%s", DCC_LOSTIDENT);
     killsock(dcc[i].sock);
     lostdcc(i);
     return;
@@ -2352,8 +2375,10 @@ static void dcc_telnet_got_ident(int i, char *host)
   /* Note: we don't really care about telnet status here. We use the
    * STATUS option as a hopefully harmless way to detect if the other
    * side is a telnet client or not. */
-  dprintf(i, TLN_IAC_C TLN_WILL_C TLN_STATUS_C);
-
+#ifdef TLS
+  if (!dcc[i].ssl)
+    dprintf(i, TLN_IAC_C TLN_WILL_C TLN_STATUS_C);
+#endif
   /* Copy acceptable-nick/host mask */
   dcc[i].status = STAT_TELNET | STAT_ECHO;
   if (!strcmp(dcc[idx].nick, "(bots)"))
@@ -2374,7 +2399,7 @@ static void dcc_telnet_got_ident(int i, char *host)
     /* Show here so it doesn't interfere with newline-less stealth_prompt */
     if (allow_new_telnets)
       dprintf(i, "(If you are new, enter 'NEW' here.)\n");
-    dprintf(i, stealth_prompt);
+    dprintf(i, "%s", stealth_prompt);
   } else {
     dprintf(i, "\n\n");
     sub_lang(i, MISC_BANNER);
