@@ -332,7 +332,10 @@ typedef intptr_t (*Function) ();
 typedef int (*IntFunc) ();
 
 #ifdef IPV6
-#include "compat/in6.h"
+  #include "compat/in6.h"
+  #define EGG_INET_ADDRSTRLEN INET6_ADDRSTRLEN
+#else
+  #define EGG_INET_ADDRSTRLEN INET_ADDRSTRLEN
 #endif
 
 #include <sys/socket.h>
@@ -376,6 +379,7 @@ struct userrec;
 
 struct dcc_t {
   long sock;                    /* This should be a long to keep 64-bit machines sane. */
+                                /* ^-- Disagreed with, but changing back to int would break ABI */
   IP addr;                      /* IP address in host network byte order. */
   sockname_t sockname;          /* IPv4/IPv6 sockaddr placeholder */
   unsigned int port;
@@ -783,6 +787,7 @@ enum {
 #endif
 
 #ifdef EGG_TDNS
+#include <pthread.h>
 #define DTN_TYPE_HOSTBYIP 0
 #define DTN_TYPE_IPBYHOST 1
 
