@@ -1133,7 +1133,7 @@ static int flag2str(char *string, int bot, int udef)
     x++;
   }
   if (string == old)
-    *string++ = '-';
+    *string++ = '*';
   return string - old;
 }
 
@@ -1494,5 +1494,6 @@ struct user_entry_type USERENTRY_BOTFL = {
   botfl_tcl_set,
   botfl_expmem,
   botfl_display,
-  "BOTFL"
+  "BOTFL",
+  NULL
 };
