@@ -1,11 +1,9 @@
 /*
  * misc_file.h
  *   prototypes for misc_file.c
- *
- * $Id: misc_file.h,v 1.13 2011/02/13 14:19:33 simple Exp $
  */
 /*
- * Copyright (C) 2000 - 2011 Eggheads Development Team
+ * Copyright (C) 2000 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,6 +24,8 @@
 #define _EGG_MISC_FILE_H
 
 int copyfile(char *, char *);
+int copyfilef(char *, FILE *);
+int fcopyfile(FILE *, char *);
 int movefile(char *, char *);
 int file_readable(char *);
 
