@@ -4,7 +4,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -151,7 +151,7 @@ static void set_mode_protect(struct chanset_t *chan, char *set)
       if (pos) {
         s1 = newsplit(&set);
         if (s1[0])
-          strncpyz(chan->key_prot, s1, sizeof chan->key_prot);
+          strlcpy(chan->key_prot, s1, sizeof chan->key_prot);
       }
       break;
     }
@@ -172,7 +172,7 @@ static void set_mode_protect(struct chanset_t *chan, char *set)
 
 static void get_mode_protect(struct chanset_t *chan, char *s)
 {
-  char *p = s, s1[121];
+  char *p = s, s1[122];
   int i, tst;
 
   s1[0] = 0;
@@ -183,11 +183,11 @@ static void get_mode_protect(struct chanset_t *chan, char *s)
         *p++ = '+';
       if (chan->limit_prot != 0) {
         *p++ = 'l';
-        sprintf(&s1[strlen(s1)], "%d ", chan->limit_prot);
+        snprintf(s1 + strlen(s1), (sizeof s1) - strlen(s1), "%d ", chan->limit_prot);
       }
       if (chan->key_prot[0]) {
         *p++ = 'k';
-        sprintf(&s1[strlen(s1)], "%s ", chan->key_prot);
+        snprintf(s1 + strlen(s1), (sizeof s1) - strlen(s1), "%s ", chan->key_prot);
       }
     } else {
       tst = chan->mode_mns_prot;
@@ -263,7 +263,7 @@ static int ismasked(masklist *m, char *user)
 
 /* Unlink chanset element from chanset list.
  */
-static inline int chanset_unlink(struct chanset_t *chan)
+static int chanset_unlink(struct chanset_t *chan)
 {
   struct chanset_t *c, *c_old = NULL;
 
@@ -289,7 +289,7 @@ static void remove_channel(struct chanset_t *chan)
   int i;
   module_entry *me;
 
-  /* Remove the channel from the list, so that noone can pull it
+  /* Remove the channel from the list, so that no one can pull it
    * away from under our feet during the check_tcl_part() call. */
   (void) chanset_unlink(chan);
 
@@ -380,19 +380,19 @@ static char *convert_element(char *src, char *dst)
  * Note:
  *  - We write chanmode "" too, so that the bot won't use default-chanmode
  *    instead of ""
- *  - We will write empty need-xxxx too, why not? (less code + lazyness)
+ *  - We will write empty need-xxxx too, why not? (less code + laziness)
  */
 static void write_channels()
 {
   FILE *f;
-  char s[121], w[1024], w2[1024], name[163];
+  char s[sizeof chanfile + 4], w[1024], w2[1024], name[163];
   char need1[242], need2[242], need3[242], need4[242], need5[242];
   struct chanset_t *chan;
   struct udef_struct *ul;
 
   if (!chanfile[0])
     return;
-  sprintf(s, "%s~new", chanfile);
+  egg_snprintf(s, sizeof s, "%s~new", chanfile);
   f = fopen(s, "w");
   chmod(s, userfile_perm);
   if (f == NULL) {
@@ -505,7 +505,7 @@ static void read_channels(int create, int reload)
   if (!readtclprog(chanfile) && create) {
     FILE *f;
 
-    /* Assume file isnt there & therfore make it */
+    /* Assume file isnt there & therefore make it */
     putlog(LOG_MISC, "*", "Creating channel file");
     f = fopen(chanfile, "w");
     if (!f)
@@ -528,7 +528,7 @@ static void read_channels(int create, int reload)
 
 static void backup_chanfile()
 {
-  char s[125];
+  char s[sizeof chanfile + 4];
 
   if (quiet_save < 2)
     putlog(LOG_MISC, "*", "Backing up channel file...");
@@ -591,9 +591,8 @@ static void channels_report(int idx, int details)
       get_mode_protect(chan, s2);
 
       if (s2[0]) {
-        s1[0] = 0;
-        sprintf(s1, ", enforcing \"%s\"", s2);
-        strcat(s, s1);
+        int len = strlen(s);
+        egg_snprintf(s + len, (sizeof s) - len, ", enforcing \"%s\"", s2); /* Concatenation */
       }
 
       s2[0] = 0;
@@ -606,18 +605,15 @@ static void channels_report(int idx, int details)
         strcat(s2, "bitch, ");
 
       if (s2[0]) {
+        int len = strlen(s);
         s2[strlen(s2) - 2] = 0;
-
-        s1[0] = 0;
-        sprintf(s1, " (%s)", s2);
-        strcat(s, s1);
+        egg_snprintf(s + len, (sizeof s) - len, " (%s)", s2); /* Concatenation */
       }
 
       /* If it's a !chan, we want to display it's unique name too <cybah> */
       if (chan->dname[0] == '!') {
-        s1[0] = 0;
-        sprintf(s1, ", unique name %s", chan->name);
-        strcat(s, s1);
+        int len = strlen(s);
+        egg_snprintf(s + len, (sizeof s) - len, ", unique name %s", chan->name); /* Concatenation */
       }
     }
 
@@ -678,7 +674,7 @@ static void channels_report(int idx, int details)
       if (channel_inactive(chan))
         i += my_strcpy(s + i, "inactive ");
       if (channel_nodesynch(chan))
-        i += my_strcpy(s + i, "nodesynch ");
+        my_strcpy(s + i, "nodesynch ");
 
       dprintf(idx, "      Options: %s\n", s);
 
@@ -828,7 +824,7 @@ static tcl_ints my_tcl_ints[] = {
   {"global-exempt-time",      &global_exempt_time,      0},
   {"global-invite-time",      &global_invite_time,      0},
   {"global-ban-type",         &global_ban_type,         0},
-  /* keeping [ban|exempt|invite]-time for compatability <Wcc[07/20/02]> */
+  /* keeping [ban|exempt|invite]-time for compatibility <Wcc[07/20/02]> */
   {"ban-time",                &global_ban_time,         0},
   {"exempt-time",             &global_exempt_time,      0},
   {"invite-time",             &global_invite_time,      0},
