@@ -429,7 +429,7 @@ static int u_addban(struct chanset_t *chan, char *ban, char *from, char *note,
     simple_sprintf(s, "%s!%s", me->funcs[SERVER_BOTNAME],
                    me->funcs[SERVER_BOTUSERHOST]);
     if (match_addr(host, s)) {
-      putlog(LOG_MISC, "*", IRC_IBANNEDME);
+      putlog(LOG_MISC, "*", "%s", IRC_IBANNEDME);
       return 0;
     }
   }
@@ -1211,7 +1211,7 @@ static void channels_writeuserfile(void)
     fclose(f);
   }
   if (ret < 3)
-    putlog(LOG_MISC, "*", USERF_ERRWRITE);
+    putlog(LOG_MISC, "*", "%s", USERF_ERRWRITE);
   write_channels();
 }
 
