@@ -143,7 +143,7 @@ static int ctcp_CHAT(char *nick, char *uhost, char *handle, char *object,
   struct userrec *u = get_user_by_handle(userlist, handle);
   int atr = u ? u->flags : 0, i;
   int chatv = AF_UNSPEC;
-  char s[INET6_ADDRSTRLEN];
+  char s[EGG_INET_ADDRSTRLEN];
 #ifdef TLS
   int ssl = 0;
 #endif
@@ -241,8 +241,8 @@ static tcl_strings mystrings[] = {
 };
 
 static tcl_ints myints[] = {
-  {"ctcp-mode", &ctcp_mode},
-  {NULL,              NULL}
+  {"ctcp-mode", &ctcp_mode, 0},
+  {NULL,              NULL, 0}
 };
 
 static char *ctcp_close()
