@@ -5,9 +5,10 @@
  *
  * Modified/written by Fabian Knittel <fknittel@gmx.de>
  * IPv6 support added by pseudo <pseudo@egg6.net>
+ * /etc/hosts support added by Michael Ortmann
  */
 /*
- * Portions Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Portions Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -38,9 +39,18 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <arpa/nameser.h>
+#undef answer /* before resolv.h because it could collide with src/mod/module.h
+                 (dietlibc) */
 #include <resolv.h>
 #include <errno.h>
 
+/* OpenBSD */
+#ifndef NS_GET16
+#define NS_GET16 GETSHORT
+#endif
+#ifndef NS_GET32
+#define NS_GET32 GETLONG
+#endif
 
 /* Defines */
 
@@ -146,14 +156,6 @@ typedef struct {
   uint16_t arcount;            /* Resource reference record count */
 } packetheader;
 
-typedef struct {
-  uint16_t datatype;
-  uint16_t class;
-  uint32_t ttl;
-  uint16_t datalength;
-  uint8_t data[FLEXIBLE_ARRAY_MEMBER];
-} res_record;
-
 #ifndef HFIXEDSZ
 #define HFIXEDSZ (sizeof(packetheader))
 #endif
@@ -170,13 +172,6 @@ typedef struct {
 #define getheader_pr(x) ((x->databyte_b >> 6) & 1)
 #define getheader_ra(x) (x->databyte_b >> 7)
 
-#define sucknetword(x)  ((x)+=2,((uint16_t)  (((x)[-2] <<  8) | ((x)[-1] <<  0))))
-#define sucknetshort(x) ((x)+=2,((short) (((x)[-2] <<  8) | ((x)[-1] <<  0))))
-#define sucknetdword(x) ((x)+=4,((dword) (((x)[-4] << 24) | ((x)[-3] << 16) | \
-                                          ((x)[-2] <<  8) | ((x)[-1] <<  0))))
-#define sucknetlong(x)  ((x)+=4,((long)  (((x)[-4] << 24) | ((x)[-3] << 16) | \
-                                          ((x)[-2] <<  8) | ((x)[-1] <<  0))))
-
 static uint32_t resrecvbuf[(MAX_PACKETSIZE + 7) >> 2]; /* MUST BE DWORD ALIGNED */
 
 static struct resolve *idbash[BASH_SIZE];
@@ -204,6 +199,18 @@ static char sendstring[1024 + 1];
 
 static const char nullstring[] = "";
 
+#ifdef res_ninit
+#define MY_RES_INIT() res_ninit(&myres);
+#define RES_MKQUERY(a, b, c, d, e, f, g, h, i) \
+    res_nmkquery(&myres, a, b, c, d, e, f, g, h, i)
+struct __res_state myres;
+#else
+#define MY_RES_INIT() res_init();
+#define RES_MKQUERY(a, b, c, d, e, f, g, h, i) \
+    res_mkquery(a, b, c, d, e, f, g, h, i)
+#define myres _res
+#endif
+
 
 /*
  *    Miscellaneous helper functions
@@ -357,12 +364,12 @@ static void linkresolvehost(struct resolve *addrp)
   rp = hostbash[bashnum];
   if (rp) {
     while ((rp->nexthost) &&
-           (egg_strcasecmp(addrp->hostn, rp->nexthost->hostn) < 0))
+           (strcasecmp(addrp->hostn, rp->nexthost->hostn) < 0))
       rp = rp->nexthost;
     while ((rp->previoushost) &&
-           (egg_strcasecmp(addrp->hostn, rp->previoushost->hostn) > 0))
+           (strcasecmp(addrp->hostn, rp->previoushost->hostn) > 0))
       rp = rp->previoushost;
-    ret = egg_strcasecmp(addrp->hostn, rp->hostn);
+    ret = strcasecmp(addrp->hostn, rp->hostn);
     if (ret < 0) {
       addrp->previoushost = rp;
       addrp->nexthost = rp->nexthost;
@@ -603,12 +610,12 @@ static struct resolve *findhost(char *hostn)
   rp = hostbash[bashnum];
   if (rp) {
     while ((rp->nexthost) &&
-          (egg_strcasecmp(hostn, rp->nexthost->hostn) >= 0))
+          (strcasecmp(hostn, rp->nexthost->hostn) >= 0))
       rp = rp->nexthost;
     while ((rp->previoushost) &&
-           (egg_strcasecmp(hostn, rp->previoushost->hostn) <= 0))
+           (strcasecmp(hostn, rp->previoushost->hostn) <= 0))
       rp = rp->previoushost;
-    if (egg_strcasecmp(hostn, rp->hostn))
+    if (strcasecmp(hostn, rp->hostn))
       return NULL;
     else {
       hostbash[bashnum] = rp;
@@ -663,7 +670,7 @@ static struct resolve *findip(IP ip)
   return rp;                    /* NULL */
 }
 
-void ptrstring4(IP *ip, char *buf, size_t sz)
+static void ptrstring4(IP *ip, char *buf, size_t sz)
 {
   egg_snprintf(buf, sz, "%u.%u.%u.%u.in-addr.arpa",
            ((uint8_t *) ip)[3],
@@ -673,7 +680,7 @@ void ptrstring4(IP *ip, char *buf, size_t sz)
 }
 
 #ifdef IPV6
-void ptrstring6(struct in6_addr *ip6, char *buf, size_t sz)
+static void ptrstring6(struct in6_addr *ip6, char *buf, size_t sz)
 {
   int i;
   char *p, *q;
@@ -686,13 +693,12 @@ void ptrstring6(struct in6_addr *ip6, char *buf, size_t sz)
      *p++ = '.';
      *p++ = hex[(ip6->s6_addr[i] >> 4) & 0x0f];
      *p++ = '.';
-     *p = '\0';
   }
   strcpy(p, "ip6.arpa"); /* ip6.int is deprecated */
 }
 #endif
 
-void ptrstring(struct sockaddr *addr, char *buf, size_t sz)
+static void ptrstring(struct sockaddr *addr, char *buf, size_t sz)
 {
   if (addr->sa_family == AF_INET)
     ptrstring4((IP *) &((struct sockaddr_in *)addr)->sin_addr.s_addr,
@@ -724,16 +730,17 @@ static void dorequest(char *s, int type, uint16_t id)
    * CPUs.
    */
   buf = nmalloc(MAX_PACKETSIZE + 1);
-  r = res_mkquery(QUERY, s, C_IN, type, NULL, 0, NULL, buf, MAX_PACKETSIZE);
+  r = RES_MKQUERY(QUERY, s, C_IN, type, NULL, 0, NULL, buf, MAX_PACKETSIZE);
   if (r == -1) {
+    nfree(buf);
     ddebug0(RES_ERR "Query too large.");
     return;
   }
   hp = (packetheader *) buf;
   hp->id = id;                  /* htons() deliberately left out (redundant) */
-  for (i = 0; i < _res.nscount; i++)
+  for (i = 0; i < myres.nscount; i++)
     (void) sendto(resfd, buf, r, 0,
-                  (struct sockaddr *) &_res.nsaddr_list[i],
+                  (struct sockaddr *) &myres.nsaddr_list[i],
                   sizeof(struct sockaddr));
   nfree(buf);
 }
@@ -817,17 +824,18 @@ static void passrp(struct resolve *rp, long ttl, int type)
 
 /* Parses the response packets received.
  */
-void parserespacket(uint8_t *response, int len)
+static void parserespacket(uint8_t *response, int len)
 {
 #ifdef IPV6
   int ready = 0;
 #endif
   int r, rcount;
-  res_record *rr;
   packetheader *hdr;
   struct resolve *rp;
-  uint8_t rc, *c = response;
-  uint16_t qdatatype, qclass;
+  uint8_t rc, *c = response, *data;
+  uint16_t qdatatype, qclass, datatype, class, datalength;
+  uint32_t ttl;
+
   if (len < sizeof(packetheader)) {
     debug1(RES_ERR "Packet smaller than standard header size: %d bytes.", len);
     return;
@@ -882,7 +890,7 @@ void parserespacket(uint8_t *response, int len)
       ptrstring(&rp->sockname.addr.sa, stackstring, sizeof stackstring);
       break;
     case STATE_AREQ:
-      strncpy(stackstring, rp->hostn, 1024);
+      strlcpy(stackstring, rp->hostn, sizeof stackstring);
   }
   c = response + HFIXEDSZ;
   r = dn_expand(response, response + len, c, namestring, MAXDNAME);
@@ -890,7 +898,7 @@ void parserespacket(uint8_t *response, int len)
     ddebug0(RES_ERR "dn_expand() failed while expanding query domain.");
     return;
   }
-  if (egg_strcasecmp(stackstring, namestring)) {
+  if (strcasecmp(stackstring, namestring)) {
     ddebug2(RES_MSG "Unknown query packet dropped. (\"%s\" does not "
             "match \"%s\")", stackstring, namestring);
     return;
@@ -900,8 +908,8 @@ void parserespacket(uint8_t *response, int len)
     ddebug0(RES_ERR "Query resource record truncated.");
     return;
   }
-  qdatatype = sucknetword(c);
-  qclass = sucknetword(c);
+  NS_GET16(qdatatype, c);
+  NS_GET16(qclass, c);
   if (qclass != C_IN) {
     ddebug2(RES_ERR "Received unsupported query class: %u (%s)",
             qclass, (qclass < CLASSTYPES_COUNT) ?
@@ -958,74 +966,74 @@ void parserespacket(uint8_t *response, int len)
       ddebug0(RES_ERR "Resource record truncated.");
       return;
     }
-    rr = ((res_record *) c);
-    rr->datatype = ntohs(rr->datatype);
-    rr->class = ntohs(rr->class);
-    rr->ttl = ntohl(rr->ttl);
-    rr->datalength = ntohs(rr->datalength);
-    if (rr->class != qclass) {
+    NS_GET16(datatype, c);
+    NS_GET16(class, c);
+    if (class != qclass) {
       ddebug2(RES_ERR "Answered class (%s) does not match queried class (%s).",
-              (rr->class < CLASSTYPES_COUNT) ?
-              classtypes[rr->class] : classtypes[CLASSTYPES_COUNT],
+              (class < CLASSTYPES_COUNT) ?
+              classtypes[class] : classtypes[CLASSTYPES_COUNT],
               (qclass < CLASSTYPES_COUNT) ?
               classtypes[qclass] : classtypes[CLASSTYPES_COUNT]);
       return;
     }
-    c += 10 + rr->datalength;
-    if (c > response + len) {
+    NS_GET32(ttl, c);
+    NS_GET16(datalength, c);
+    data = c;
+    c += datalength;
+    if (c > (response + len)) {
       ddebug0(RES_ERR "Specified rdata length exceeds packet size.");
       return;
     }
-    if (egg_strcasecmp(stackstring, namestring))
+    if (strcasecmp(stackstring, namestring))
       continue;
-    if (rr->datatype != qdatatype && rr->datatype != T_CNAME) {
+    if (datatype != qdatatype && datatype != T_CNAME) {
       ddebug2(RES_MSG "Ignoring resource type %u. (%s)",
-              rr->datatype, (rr->datatype < RESOURCETYPES_COUNT) ?
-              resourcetypes[rr->datatype] :
+              datatype, (datatype < RESOURCETYPES_COUNT) ?
+              resourcetypes[datatype] :
               resourcetypes[RESOURCETYPES_COUNT]);
       continue;
     }
-    ddebug1(RES_MSG "TTL: %s", strtdiff(sendstring, rr->ttl));
-    ddebug1(RES_MSG "TYPE: %s", (rr->datatype < RESOURCETYPES_COUNT) ?
-            resourcetypes[rr->datatype] : resourcetypes[RESOURCETYPES_COUNT]);
-    switch (rr->datatype) {
+    ddebug1(RES_MSG "TTL: %s", strtdiff(sendstring, ttl));
+    ddebug1(RES_MSG "TYPE: %s", (datatype < RESOURCETYPES_COUNT) ?
+            resourcetypes[datatype] : resourcetypes[RESOURCETYPES_COUNT]);
+    switch (datatype) {
       case T_A:
-        if (rr->datalength != 4) {
+        if (datalength != 4) {
           ddebug1(RES_ERR "Unsupported rdata format for \"A\" type. "
-                  "(%u bytes)", rr->datalength);
+                  "(%u bytes)", datalength);
           return;
         }
-        rp->ttl = rr->ttl;
+        rp->ttl = ttl;
         rp->sockname.addrlen = sizeof(struct sockaddr_in);
         rp->sockname.addr.sa.sa_family = AF_INET;
-        memcpy(&rp->sockname.addr.s4.sin_addr, rr->data, 4);
+        memcpy(&rp->sockname.addr.s4.sin_addr, data, 4);
 #ifndef IPV6
-        passrp(rp, rr->ttl, T_A);
+        passrp(rp, ttl, T_A);
         return;
 #else
         if (ready || !pref_af) {
-          passrp(rp, rr->ttl, T_A);
+          passrp(rp, ttl, T_A);
           return;
         }
         break;
       case T_AAAA:
-        if (rr->datalength != 16) {
+        if (datalength != 16) {
           ddebug1(RES_ERR "Unsupported rdata format for \"AAAA\" type. "
-                  "(%u bytes)", rr->datalength);
+                  "(%u bytes)", datalength);
           return;
         }
-        rp->ttl = rr->ttl;
+        rp->ttl = ttl;
         rp->sockname.addrlen = sizeof(struct sockaddr_in6);
         rp->sockname.addr.sa.sa_family = AF_INET6;
-        memcpy(&rp->sockname.addr.s6.sin6_addr, rr->data, 16);
+        memcpy(&rp->sockname.addr.s6.sin6_addr, data, 16);
         if (ready || pref_af) {
-          passrp(rp, rr->ttl, T_A);
+          passrp(rp, ttl, T_A);
           return;
         }
         break;
 #endif
       case T_PTR:
-        r = dn_expand(response, response + len, rr->data, namestring, MAXDNAME);
+        r = dn_expand(response, response + len, data, namestring, MAXDNAME);
         if (r == -1) {
           ddebug0(RES_ERR "dn_expand() failed while expanding domain in "
                   "rdata.");
@@ -1041,12 +1049,12 @@ void parserespacket(uint8_t *response, int len)
           rp->hostn = nmalloc(strlen(namestring) + 1);
           strcpy(rp->hostn, namestring);
           linkresolvehost(rp);
-          passrp(rp, rr->ttl, T_PTR);
+          passrp(rp, ttl, T_PTR);
           return;
         }
         break;
       case T_CNAME:
-        r = dn_expand(response, response + len, rr->data, namestring, MAXDNAME);
+        r = dn_expand(response, response + len, data, namestring, MAXDNAME);
         if (r == -1) {
           ddebug0(RES_ERR "dn_expand() failed while expanding domain in "
                   "rdata.");
@@ -1057,8 +1065,8 @@ void parserespacket(uint8_t *response, int len)
         break;
       default:
         ddebug2(RES_ERR "Received unimplemented data type: %u (%s)",
-                rr->datatype, (rr->datatype < RESOURCETYPES_COUNT) ?
-                resourcetypes[rr->datatype] :
+                datatype, (datatype < RESOURCETYPES_COUNT) ?
+                resourcetypes[datatype] :
                 resourcetypes[RESOURCETYPES_COUNT]);
     }
   }
@@ -1092,17 +1100,17 @@ static void dns_ack(void)
   }
   /* Check to see if this server is actually one we sent to */
   if (from.sin_addr.s_addr == localhost) {
-    for (i = 0; i < _res.nscount; i++)
+    for (i = 0; i < myres.nscount; i++)
       /* 0.0.0.0 replies as 127.0.0.1 */
-      if ((_res.nsaddr_list[i].sin_addr.s_addr == from.sin_addr.s_addr) ||
-          (!_res.nsaddr_list[i].sin_addr.s_addr))
+      if ((myres.nsaddr_list[i].sin_addr.s_addr == from.sin_addr.s_addr) ||
+          (!myres.nsaddr_list[i].sin_addr.s_addr))
         break;
   } else {
-    for (i = 0; i < _res.nscount; i++)
-      if (_res.nsaddr_list[i].sin_addr.s_addr == from.sin_addr.s_addr)
+    for (i = 0; i < myres.nscount; i++)
+      if (myres.nsaddr_list[i].sin_addr.s_addr == from.sin_addr.s_addr)
         break;
   }
-  if (i == _res.nscount)
+  if (i == myres.nscount)
     ddebug1(RES_ERR "Received reply from unknown source: %s",
                 iptostr((struct sockaddr *) &from));
   else
@@ -1196,6 +1204,112 @@ static void dns_lookup(sockname_t *addr)
   sendrequest(rp, T_PTR);
 }
 
+/* Read /etc/hosts */
+static int dns_hosts(char *hostn) {
+  #define PATH "/etc/hosts"
+  size_t hostn_len;
+  int i;
+  char hostn_lower[256], hostn_upper[256], line[8 * 1024], *p1, *p2, *p3, *p4;
+  FILE *hostf;
+  sockname_t name;
+#ifdef IPV6
+  int af, fallback_set = 0;
+  char fallback_ip[sizeof line];
+#endif
+
+  if (!*hostn) {
+    ddebug0(RES_MSG "ERROR: Bogus empty hostname input");
+    return 1;
+  }
+  hostn_len = strlen(hostn);
+  if (hostn_len > 255) {
+    ddebug0(RES_MSG "ERROR: Bogus len of hostname > 255 input");
+    return 1;
+  }
+  /* precalculate lower and upper string from hostn and compare with handcrafted
+   * code, due to strncasecmp() and strncmp() being slow if used in loop */
+  for (i = 0; i < hostn_len; i++) {
+      /* while at it, reject hostnames with bogus chars, see rfc 952, 1123 and 2181 */
+      if (!strchr("-.0123456789ABCDEFGHIJABCDEFGHIJKLMNOPQRSTUVWXYZ_abcdefghijklmnopqrstuvwxyz", hostn[i])) {
+
+        ddebug2(RES_MSG "ERROR: Bogus char in hostname input: 0x%02x at %i", hostn[i], i + 1);
+        return 1;
+      }
+      hostn_lower[i] = tolower((unsigned char) hostn[i]);
+      hostn_upper[i] = toupper((unsigned char) hostn[i]);
+  }
+  hostn_lower[i] = 0;
+  hostn_upper[i] = 0;
+  if (!(hostf = fopen(PATH, "r"))) {
+    ddebug1(RES_MSG "WARNING: fopen(" PATH "): %s", strerror(errno));
+    return 0;
+  }
+  /* p1 is used for finding ip
+   * p2 is used for finding hostname
+   * p3 and p4 are used to compare against hostn_lower and hostn_upper
+   */
+  while ((p1 = fgets(line, sizeof line, hostf))) {
+    /* postel's law */
+    while ((*p1 == ' ') || (*p1 == '\t'))
+      p1++;
+    p2 = p1;
+    while ((*p2 != '\n') && (*p2 != '#') && (*p2 != '\0')) {
+      if ((*p2 == ' ') || (*p2 == '\t')) {
+        *p2++ = '\0'; /* null-terminate ip */
+        /* skip tab and space */
+        while ((*p2 == ' ') || (*p2 == '\t'))
+          p2++;
+        /* if (!strncasecmp(c, hostn, hostn_len)) { */
+        p3 = hostn_lower;
+        p4 = hostn_upper;
+        while ((*p2 == *p3) || (*p2 == *p4) ||
+               ((*p3 == '\0') && ((*p2 == '\n') || (*p2 == ' ') || (*p2 == '\t') || (*p2 == '#')))) {
+          if (*p3 == '\0') {
+            /* string compare was successful, found hostname */
+#ifdef IPV6
+            if ((af = setsockname(&name, p1, 0, 0)) == (pref_af ? AF_INET6 : AF_INET)) {
+#else
+            if ((strchr(p1, '.')) && (setsockname(&name, p1, 0, 0) != AF_UNSPEC)) {
+#endif
+              call_ipbyhost(hostn, &name, 1);
+              ddebug2(RES_MSG "Used /etc/hosts: %s == %s", hostn, p1);
+              fclose(hostf);
+              return 1;
+            }
+#ifdef IPV6
+            else if ((af != AF_UNSPEC) && !fallback_set) {
+              fallback_set = 1;
+              strlcpy(fallback_ip, line, sizeof fallback_ip);
+            }
+#endif
+            p2++;
+            break;
+          }
+          p2++;
+          p3++;
+          p4++;
+        }
+      }
+      p2++;
+    }
+  }
+  if (ferror(hostf)) {
+    ddebug0(RES_MSG "ERROR: fgets(" PATH ")");
+    fclose(hostf);
+    return 0;
+  }
+#ifdef IPV6
+  if (fallback_set) {
+    call_ipbyhost(hostn, &name, 1);
+    ddebug2(RES_MSG "Used /etc/hosts: %s == %s", hostn, fallback_ip);
+    fclose(hostf);
+    return 1;
+  }
+#endif
+  fclose(hostf);
+  return 0;
+}
+
 /* Start searching for an ip-address, using it's host-name.
  */
 static void dns_forward(char *hostn)
@@ -1220,6 +1334,8 @@ static void dns_forward(char *hostn)
     }
     return;
   }
+  if (dns_hosts(hostn))
+    ddebug0(RES_MSG "Couldnt lookup /etc/hosts");
   ddebug0(RES_MSG "Creating new record");
   rp = allocresolve();
   rp->state = STATE_AREQ;
@@ -1235,8 +1351,7 @@ static void dns_forward(char *hostn)
  */
 static int init_dns_network(void)
 {
-  int option;
-  struct in_addr inaddr;
+  int option = 1;
 
   resfd = socket(AF_INET, SOCK_DGRAM, 0);
   if (resfd == -1) {
@@ -1251,18 +1366,16 @@ static int init_dns_network(void)
     killsock(resfd);
     return 0;
   }
-  option = 1;
-  if (setsockopt(resfd, SOL_SOCKET, SO_BROADCAST, (char *) &option,
-                 sizeof(option))) {
-    putlog(LOG_MISC, "*",
-           "Unable to setsockopt() on nameserver communication socket: %s",
-           strerror(errno));
-    killsock(resfd);
-    return 0;
+  if (setsockopt(resfd, SOL_SOCKET, SO_BROADCAST, &option, sizeof option)) {
+    if (errno != ENOSYS) {
+      putlog(LOG_MISC, "*",
+             "Unable to setsockopt() on nameserver communication socket: %s",
+             strerror(errno));
+      killsock(resfd);
+      return 0;
+    }
   }
-
-  egg_inet_aton("127.0.0.1", &inaddr);
-  localhost = inaddr.s_addr;
+  localhost = htonl(INADDR_LOOPBACK);
   return 1;
 }
 
@@ -1273,17 +1386,14 @@ static int init_dns_core(void)
   int i;
 
   /* Initialise the resolv library. */
-  res_init();
-  #ifdef NETBSD_HACKS
-    puts("netbsd found. if eggdrop crashes with\n"
-         "  _res is not supported for multi-threaded programs.\n"
-         "  [1]   Abort trap (core dumped) ./eggdrop -nt\n"
-         "dns.mod must be disabled by commenting out in eggdrop.conf\n"
-         "  #loadmodule dns");
-  #endif
-  _res.options |= RES_RECURSE | RES_DEFNAMES | RES_DNSRCH;
-  for (i = 0; i < _res.nscount; i++)
-    _res.nsaddr_list[i].sin_family = AF_INET;
+  MY_RES_INIT();
+  if (!myres.nscount) {
+    putlog(LOG_MISC, "*", "No nameservers defined.");
+    return 0;
+  }
+  myres.options |= RES_RECURSE | RES_DEFNAMES | RES_DNSRCH;
+  for (i = 0; i < myres.nscount; i++)
+    myres.nsaddr_list[i].sin_family = AF_INET;
 
   if (!init_dns_network())
     return 0;
