@@ -659,7 +659,7 @@ static void cmd_kick(struct userrec *u, int idx, char *par)
     return;
   }
   if (!me_op(chan) && chan_hasop(m)) {
-    dprintf(idx, "I can't help you now because halfops cannot kick ops.\n",
+    dprintf(idx, "I can't help you now because halfops cannot kick ops (on %s).\n",
             chan->dname);
     return;
   }
@@ -724,21 +724,8 @@ static void cmd_channel(struct userrec *u, int idx, char *par)
 {
   char handle[HANDLEN + 1], s[UHOSTLEN], s1[UHOSTLEN], atrflag, chanflag;
   struct chanset_t *chan;
-  struct capability *current;
   memberlist *m;
-  int maxnicklen, maxhandlen, extjoin = 0, acctnotify = 0;
-
-  /* Check if CAPs are enabled */
-  current = cap;
-  while (current != NULL) {
-    if (!strcasecmp("extended-join", current->name)) {
-      extjoin = current->enabled ? 1 : 0;
-    }
-    if (!strcasecmp("account-notify", current->name)) {
-      acctnotify = current->enabled ? 1 : 0;
-    }
-    current = current->next;
-  }
+  int maxnicklen, maxhandlen;
 
   chan = get_channel(idx, par);
   if (!chan || !has_oporhalfop(idx, chan))
@@ -770,14 +757,9 @@ static void cmd_channel(struct userrec *u, int idx, char *par)
       maxhandlen = 9;
 
     dprintf(idx, "(n = owner, m = master, o = op, d = deop, b = bot)\n");
-    if (use_354 && extjoin && acctnotify) {
-      dprintf(idx, " %-*s %-*s %-*s  %-6s %-5s %s\n", maxnicklen, "NICKNAME",
-                maxhandlen, "HANDLE", maxnicklen, "ACCOUNT", "JOIN", "IDLE",
-                "USER@HOST");
-    } else {
-      dprintf(idx, " %-*s %-*s  %-6s %-5s %s\n", maxnicklen, "NICKNAME",
-                maxhandlen, "HANDLE", "JOIN", "IDLE", "USER@HOST");
-    }
+    dprintf(idx, " %-*s %-*s %-*s  %-6s %-5s        %s\n", maxnicklen, "NICKNAME",
+              maxhandlen, "HANDLE", maxnicklen, "ACCOUNT", "JOIN", "IDLE",
+              "USER@HOST");
     for (m = chan->channel.member; m && m->nick[0]; m = m->next) {
       if (m->joined > 0) {
         if ((now - (m->joined)) > 86400)
@@ -871,24 +853,13 @@ static void cmd_channel(struct userrec *u, int idx, char *par)
       else
         chanflag = ' ';
       if (chan_issplit(m)) {
-        if (use_354 && extjoin && acctnotify) {
-          dprintf(idx, "%c%-*s %-*s %-*s %-6s %-5s <- netsplit, %lus\n",
-                chanflag, maxnicklen, m->nick, maxhandlen, handle, maxnicklen,
-                m->account, s, atrflag, now- (m->split));
-        } else {
-          dprintf(idx, "%c%-*s %-*s %-6s %-5s <- netsplit, %lus\n", chanflag,
-                maxnicklen, m->nick, maxhandlen, handle, s, atrflag,
-                now- (m->split));
-        }
+        dprintf(idx, "%c%-*s %-*s %-*s %-6s %c             <- netsplit, %fs\n",
+              chanflag, maxnicklen, m->nick, maxhandlen, handle, maxnicklen,
+              m->account, s, atrflag, difftime(now, m->split));
       } else if (!rfc_casecmp(m->nick, botname)) {
-        if (use_354 && extjoin && acctnotify) {
-          dprintf(idx, "%c%-*s %-*s %-*s %-6s %c     <- it's me!\n", chanflag,
-                maxnicklen, m->nick, maxhandlen, handle, maxnicklen, m->account,
-                s, atrflag);
-        } else {
-          dprintf(idx, "%c%-*s %-*s %-6s %c     <- it's me!\n", chanflag,
-                maxnicklen, m->nick, maxhandlen, handle, s, atrflag);
-        }
+        dprintf(idx, "%c%-*s %-*s %-*s %-6s %c             <- it's me!\n", chanflag,
+              maxnicklen, m->nick, maxhandlen, handle, maxnicklen, m->account,
+              s, atrflag);
       } else {
         /* Determine idle time */
         if (now - (m->last) > 86400)
@@ -900,18 +871,13 @@ static void cmd_channel(struct userrec *u, int idx, char *par)
         else
           strlcpy(s1, "   ", sizeof s1);
         if (chan_ircaway(m)) {
-          egg_snprintf(s1+strlen(s1), ((sizeof s1)-strlen(s1)), " (away)");
+          strlcpy(s1+strlen(s1), " (away)", ((sizeof s1)-strlen(s1)));
         } else {
-          egg_snprintf(s1+strlen(s1), ((sizeof s1)-strlen(s1)), "       ");
-        }
-        if (use_354 && extjoin && acctnotify) {
-          dprintf(idx, "%c%-*s %-*s %-*s %-6s %c %s  %s\n", chanflag, maxnicklen,
-                m->nick, maxhandlen, handle, maxnicklen, m->account, s, atrflag,
-                s1, m->userhost);
-        } else {
-          dprintf(idx, "%c%-*s %-*s %-6s %c %-5s  %s\n", chanflag, maxnicklen,
-                m->nick, maxhandlen, handle, s, atrflag, s1, m->userhost);
+          strlcpy(s1+strlen(s1), "       ", ((sizeof s1)-strlen(s1)));
         }
+        dprintf(idx, "%c%-*s %-*s %-*s %-6s %c %s  %s\n", chanflag, maxnicklen,
+              m->nick, maxhandlen, handle, maxnicklen, m->account, s, atrflag,
+              s1, m->userhost);
       }
       if (chan_fakeop(m))
         dprintf(idx, "    (%s)\n", IRC_FAKECHANOP);
