@@ -33,7 +33,7 @@
 
 static p_tcl_bind_list H_topc, H_splt, H_sign, H_rejn, H_part, H_pub, H_pubm;
 static p_tcl_bind_list H_nick, H_mode, H_kick, H_join, H_need, H_invt, H_ircaway;
-static p_tcl_bind_list H_monitor;
+static p_tcl_bind_list H_monitor, H_account;
 
 static Function *global = NULL, *channels_funcs = NULL, *server_funcs = NULL;
 
@@ -460,7 +460,7 @@ void reset_chan_info(struct chanset_t *chan, int reset, int do_reset)
 }
 
 /* Leave the specified channel and notify registered Tcl procs. This
- * should not be called by itsself.
+ * should not be called by itself.
  */
 static void do_channel_part(struct chanset_t *chan)
 {
@@ -981,6 +981,22 @@ static void check_tcl_need(char *chname, char *type)
                  MATCH_MASK | BIND_STACKABLE);
 }
 
+static void check_tcl_account(char *nick, char *uhost, struct userrec *u, char *chan, char *account)
+{
+  char mask[1024];
+  struct flag_record fr = { FR_GLOBAL | FR_CHAN | FR_ANYWH, 0, 0, 0, 0, 0 };
+
+  snprintf(mask, sizeof mask, "%s %s!%s %s", chan, nick, uhost, account);
+  Tcl_SetVar(interp, "_acnt1", nick, 0);
+  Tcl_SetVar(interp, "_acnt2", uhost, 0);
+  Tcl_SetVar(interp, "_acnt3", u ? u->handle : "*", 0);
+  Tcl_SetVar(interp, "_acnt4", chan, 0);
+  Tcl_SetVar(interp, "_acnt5", account, 0);
+  check_tcl_bind(H_account, mask, &fr,
+       " $_acnt1 $_acnt2 $_acnt3 $_acnt4 $_acnt5", MATCH_MASK | BIND_STACKABLE);
+}
+
+
 static tcl_strings mystrings[] = {
   {"opchars", opchars, 7, 0},
   {NULL,      NULL,    0, 0}
@@ -1045,13 +1061,66 @@ static void flush_modes()
   }
 }
 
+static void tell_account_tracking_status(int idx, int details)
+{
+  struct capability *current;
+  int extjoin = 0, notify = 0, tag = 0, whox = use_354;
+  /* List status of account tracking. For 100% accuracy, this requires
+   * WHOX ability (354 messages) and the extended-join and account-notify
+   * capabilities to be enabled.
+   */
+  /* Check if CAPs are enabled */
+  current = cap;
+  while (current != NULL) {
+    if (!strcasecmp("extended-join", current->name) && current->enabled) {
+      extjoin = 1;
+    } else if (!strcasecmp("account-notify", current->name) && current->enabled) {
+      notify = 1;
+    } else if (!strcasecmp("account-tag", current->name) && current->enabled) {
+      tag = 1;
+    }
+    current = current->next;
+  }
+
+  if (whox && notify && extjoin) {
+    dprintf(idx, "%s", "    Account tracking: Enabled\n");
+  } else {
+    if (!details) {
+      dprintf(idx, "    Account tracking: Best-effort (Missing capabilities:%s%s%s%s)\n",
+                    whox ? "" : " WHOX", notify ? "" : " account-notify", extjoin ? "" : " extended-join",
+                    details ? "" : ", see .status all for details");
+    } else {
+      dprintf(idx, "    Account tracking: Best-effort\n");
+      if (!whox) {
+        dprintf(idx, "%s", "      - WHOX missing           => Accounts will NOT be known after Eggdrop joins a channel (raw 315)\n");
+      } else {
+        dprintf(idx, "%s", "      - WHOX enabled           => Accounts will be known after Eggdrop joins a channel (raw 315)\n");
+      }
+
+      if (!notify) {
+        dprintf(idx, "%s", "      - account-notify missing => Accounts will NOT update immediately when users log in or out\n");
+      } else {
+        dprintf(idx, "%s", "      - account-notify enabled => Accounts will update immediately when users log in or out\n");
+      }
+      if (!extjoin) {
+        dprintf(idx, "%s", "      - extended-join missing  => Accounts will NOT be known immediately when a user joins (bind join)\n");
+      } else {
+        dprintf(idx, "%s", "      - extended-join enabled  => Accounts will be known immediately when a user joins (bind join)\n");
+      }
+      if (tag && (!whox || !notify || !extjoin)) {
+        dprintf(idx, "%s", "      - account-tag enabled    => Accounts will update whenever someone messages a channel or this bot\n");
+      }
+      dprintf(idx, "%s", "      See doc/ACCOUNTS for more details\n");
+    }
+  }
+}
+
 static void irc_report(int idx, int details)
 {
   struct flag_record fr = { FR_GLOBAL | FR_CHAN, 0, 0, 0, 0, 0 };
   char ch[1024], q[256], *p;
-  int k, l, extjoin, acctnotify;
+  int k, l;
   struct chanset_t *chan;
-  struct capability *current;
 
   strcpy(q, "Channels: ");
   k = 10;
@@ -1084,33 +1153,7 @@ static void irc_report(int idx, int details)
     q[k - 2] = 0;
     dprintf(idx, "    %s\n", q);
   }
-  /* List status of account tracking. For 100% accuracy, this requires
-   * WHOX ability (354 messages) and the extended-join and account-notify
-   * capabilities to be enabled.
-   */
-  /* Check if CAPs are enabled */
-  current = cap;
-  extjoin = 0;
-  acctnotify = 0;
-  while (current != NULL) {
-    if (!strcasecmp("extended-join", current->name)) {
-      extjoin = 1;
-    }
-    if (!strcasecmp("account-notify", current->name)) {
-      acctnotify = 1;
-    }
-    current = current->next;
-  }
-
-  if (use_354 && extjoin && acctnotify) {
-    dprintf(idx, "    Account tracking: Enabled\n");
-  } else {
-    dprintf(idx, "    Account tracking: Disabled\n"
-                 "      (Missing capabilities:%s%s%s)\n",
-                      use_354 ? "" : " use-354",
-                      extjoin ? "" : " extended-join",
-                      acctnotify ? "" : " account-notify");
-  }
+  tell_account_tracking_status(idx, details);
 }
 
 /* Many networks either support max_bans/invite/exempts/ *or*
@@ -1296,12 +1339,12 @@ static char *irc_close()
   del_bind_table(H_need);
   del_bind_table(H_ircaway);
   del_bind_table(H_monitor);
+  del_bind_table(H_account);
   rem_tcl_strings(mystrings);
   rem_tcl_ints(myints);
   rem_builtins(H_dcc, irc_dcc);
   rem_builtins(H_msg, C_msg);
   rem_builtins(H_raw, irc_raw);
-  rem_builtins(H_rawt, irc_rawt);
   rem_builtins(H_isupport, irc_isupport_binds);
   rem_tcl_commands(tclchan_cmds);
   rem_help_reference("irc.help");
@@ -1428,6 +1471,7 @@ char *irc_start(Function *global_funcs)
   H_need = add_bind_table("need", HT_STACKABLE, channels_2char);
   H_ircaway = add_bind_table("ircaway", HT_STACKABLE, channels_5char);
   H_monitor = add_bind_table("monitor", HT_STACKABLE, monitor_2char);
+  H_account = add_bind_table("account", HT_STACKABLE, channels_5char);
   do_nettype();
   return NULL;
 }
