@@ -4,7 +4,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -42,7 +42,7 @@ static int msg_hello(char *nick, char *h, struct userrec *u, char *p)
       return 1;
     }
   }
-  strncpyz(handle, nick, sizeof(handle));
+  strlcpy(handle, nick, sizeof(handle));
   if (get_user_by_handle(userlist, handle)) {
     dprintf(DP_HELP, IRC_BADHOST1, nick);
     dprintf(DP_HELP, IRC_BADHOST2, nick, botname);
@@ -111,7 +111,7 @@ static int msg_hello(char *nick, char *h, struct userrec *u, char *p)
       if (p1 == NULL)
         s1[0] = 0;
       else
-        strncpyz(s1, p1, sizeof s1);
+        strlcpy(s1, p1, sizeof s1);
     }
   }
   return 1;
@@ -119,7 +119,7 @@ static int msg_hello(char *nick, char *h, struct userrec *u, char *p)
 
 static int msg_pass(char *nick, char *host, struct userrec *u, char *par)
 {
-  char *old, *new;
+  char *old, *new, *s;
 
   if (!u || match_my_nick(nick) || (u->flags & (USER_BOT | USER_COMMON)))
     return 1;
@@ -144,16 +144,13 @@ static int msg_pass(char *nick, char *host, struct userrec *u, char *par)
   } else
     new = old;
   putlog(LOG_CMDS, "*", "(%s!%s) !%s! PASS...", nick, host, u->handle);
-  if (strlen(new) > 15)
-    new[15] = 0;
-  if (strlen(new) < 6) {
-    dprintf(DP_HELP, "NOTICE %s :%s\n", nick, IRC_PASSFORMAT);
-    return 0;
+  if ((s = check_validpass(u, new))) {
+    dprintf(DP_HELP, "NOTICE %s :%s\n", nick, s);
+    return 1;
   }
-  set_user(&USERENTRY_PASS, u, new);
   dprintf(DP_HELP, "NOTICE %s :%s '%s'.\n", nick,
           new == old ? IRC_SETPASS : IRC_CHANGEPASS, new);
-  return 1;
+  return 0;
 }
 
 static int msg_ident(char *nick, char *host, struct userrec *u, char *par)
@@ -171,11 +168,9 @@ static int msg_ident(char *nick, char *host, struct userrec *u, char *par)
   }
   pass = newsplit(&par);
   if (!par[0])
-    strncpyz(who, nick, sizeof who);
-  else {
-    strncpy(who, par, NICKMAX);
-    who[NICKMAX] = 0;
-  }
+    strlcpy(who, nick, sizeof who);
+  else
+    strlcpy(who, par, sizeof who);
   u2 = get_user_by_handle(userlist, who);
   if (!u2) {
     if (u && !quiet_reject)
@@ -301,7 +296,7 @@ static int msg_info(char *nick, char *host, struct userrec *u, char *par)
       dprintf(DP_HELP, "NOTICE %s :%s\n", nick, IRC_INFOLOCKED);
       return 1;
     }
-    if (!egg_strcasecmp(par, "none")) {
+    if (!strcasecmp(par, "none")) {
       par[0] = 0;
       if (chname) {
         set_handle_chaninfo(userlist, u->handle, chname, NULL);
@@ -356,7 +351,7 @@ static int msg_who(char *nick, char *host, struct userrec *u, char *par)
   struct chanset_t *chan;
   struct flag_record fr = { FR_GLOBAL | FR_CHAN, 0, 0, 0, 0, 0 };
   memberlist *m;
-  char s[UHOSTLEN], also[512], *info;
+  char s[NICKLEN + UHOSTLEN], also[512], *info;
   int i;
 
   if (!use_info || match_my_nick(nick))
@@ -436,7 +431,7 @@ static int msg_who(char *nick, char *host, struct userrec *u, char *par)
 
 static int msg_whois(char *nick, char *host, struct userrec *u, char *par)
 {
-  char s[UHOSTLEN], s1[81], *s2;
+  char s[UHOSTLEN + 1], s1[143], *s2, stime[14];
   int ok;
   struct chanset_t *chan;
   memberlist *m;
@@ -487,7 +482,7 @@ static int msg_whois(char *nick, char *host, struct userrec *u, char *par)
   if (s2 && s2[0] && !(u2->flags & USER_BOT))
     dprintf(DP_HELP, "NOTICE %s :[%s] %s\n", nick, u2->handle, s2);
   for (xk = get_user(&USERENTRY_XTRA, u2); xk; xk = xk->next)
-    if (!egg_strcasecmp(xk->key, "EMAIL"))
+    if (!strcasecmp(xk->key, "EMAIL"))
       dprintf(DP_HELP, "NOTICE %s :[%s] E-mail: %s\n", nick, u2->handle,
               xk->data);
   ok = 0;
@@ -503,10 +498,10 @@ static int msg_whois(char *nick, char *host, struct userrec *u, char *par)
           hand_on_chan(chan, u) || (glob_op(fr) && !chan_deop(fr)) ||
           glob_friend(fr) || chan_op(fr) || chan_friend(fr))) {
         tt = cr->laston;
-        egg_strftime(s, 14, "%b %d %H:%M", localtime(&tt));
+        strftime(stime, sizeof stime, "%b %d %H:%M", localtime(&tt));
         ok = 1;
         egg_snprintf(s1, sizeof s1, "NOTICE %s :[%s] %s %s on %s", nick,
-                     u2->handle, IRC_LASTSEENAT, s, chan->dname);
+                     u2->handle, IRC_LASTSEENAT, stime, chan->dname);
       }
     }
   }
@@ -785,25 +780,11 @@ static int msg_invite(char *nick, char *host, struct userrec *u, char *par)
 
 static int msg_status(char *nick, char *host, struct userrec *u, char *par)
 {
-  char s[256], *vers_t, *uni_t, *pass;
+  char s[256], *pass, *sysrel;
   int i;
   struct chanset_t *chan;
   time_t now2 = now - online_since, hr, min;
 
-#ifdef HAVE_UNAME
-  struct utsname un;
-
-  if (uname(&un) < 0) {
-#endif
-    vers_t = " ";
-    uni_t  = "*unknown*";
-#ifdef HAVE_UNAME
-  } else {
-    vers_t = un.release;
-    uni_t  = un.sysname;
-  }
-#endif
-
   if (match_my_nick(nick))
     return 1;
 
@@ -844,12 +825,14 @@ static int msg_status(char *nick, char *host, struct userrec *u, char *par)
 
   if (admin[0])
     dprintf(DP_HELP, "NOTICE %s :Admin: %s.\n", nick, admin);
-  dprintf(DP_HELP, "NOTICE %s :OS: %s %s.\n", nick, uni_t, vers_t);
+  sysrel = egg_uname();
+  if (*sysrel)
+    dprintf(DP_HELP, "NOTICE %s :OS: %s.\n", nick, sysrel);
   dprintf(DP_HELP, "NOTICE %s :Online as: %s!%s.\n", nick, botname, botuserhost);
 
   /* This shouldn't overflow anymore -Wcc */
   s[0] = 0;
-  strncpyz(s, "Channels: ", sizeof s);
+  strlcpy(s, "Channels: ", sizeof s);
   for (chan = chanset; chan; chan = chan->next) {
     strncat(s, chan->dname, sizeof(s) - 1 - strlen(s));
     if (!channel_active(chan))
@@ -999,13 +982,13 @@ static int msg_reset(char *nick, char *host, struct userrec *u, char *par)
     }
     putlog(LOG_CMDS, "*", "(%s!%s) !%s! RESET %s", nick, host, u->handle, par);
     dprintf(DP_HELP, "NOTICE %s :%s: %s\n", nick, par, IRC_RESETCHAN);
-    reset_chan_info(chan, CHAN_RESETALL);
+    reset_chan_info(chan, CHAN_RESETALL, 1);
     return 1;
   }
   putlog(LOG_CMDS, "*", "(%s!%s) !%s! RESET ALL", nick, host, u->handle);
   dprintf(DP_HELP, "NOTICE %s :%s\n", nick, IRC_RESETCHAN);
   for (chan = chanset; chan; chan = chan->next)
-    reset_chan_info(chan, CHAN_RESETALL);
+    reset_chan_info(chan, CHAN_RESETALL, 1);
   return 1;
 }
 
