@@ -11,7 +11,7 @@
  * 1.2a    1997-08-24      Minor fixes. [BB]
  */
 /*
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -184,7 +184,7 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand,
       !oix[2]) || (!oix[1] && (oix[-1] == 's' || oix[-1] == 'z' ||
       oix[-1] == 'x' || oix[-1] == 'S' || oix[-1] == 'Z' ||
       oix[-1] == 'X')))) {
-    strncpy(object, word1, oix - word1);
+    strlcpy(object, word1, sizeof object);
     object[oix - word1] = 0;
     wordshift(word1, text);
     if (!word1[0]) {
@@ -201,7 +201,7 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand,
           onchan = 1;
           snprintf(stuff, sizeof stuff, "%s!%s", object, m->userhost);
           urec = get_user_by_host(stuff);
-          if (!urec || !egg_strcasecmp(object, urec->handle))
+          if (!urec || !strcasecmp(object, urec->handle))
             break;
           strcat(whoredirect, object);
           strcat(whoredirect, " is ");
@@ -218,7 +218,7 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand,
         return;
       }
     }
-    if (!egg_strcasecmp(word1, "bf") || !egg_strcasecmp(word1, "boyfriend")) {
+    if (!strcasecmp(word1, "bf") || !strcasecmp(word1, "boyfriend")) {
       strlcpy(whotarget, getxtra(object, "BF"), sizeof whotarget);
       if (whotarget[0]) {
         snprintf(whoredirect, sizeof whoredirect, "%s boyfriend is %s, and ",
@@ -230,7 +230,7 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand,
               prefix, fixnick(object), nick);
       return;
     }
-    if (!egg_strcasecmp(word1, "gf") || !egg_strcasecmp(word1, "girlfriend")) {
+    if (!strcasecmp(word1, "gf") || !strcasecmp(word1, "girlfriend")) {
       strlcpy(whotarget, getxtra(object, "GF"), sizeof whotarget);
       if (whotarget[0]) {
         snprintf(whoredirect, sizeof whoredirect, "%s girlfriend is %s, and ",
@@ -248,7 +248,7 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand,
     return;
   }
   /* Keyword "my" */
-  if (!egg_strcasecmp(word1, "my")) {
+  if (!strcasecmp(word1, "my")) {
     wordshift(word1, text);
     if (!word1[0]) {
       dprintf(idx, "%sYour what, %s?\n", prefix, nick);
@@ -262,7 +262,7 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand,
       return;
     }
     /* "my boyfriend" */
-    if (!egg_strcasecmp(word1, "boyfriend") || !egg_strcasecmp(word1, "bf")) {
+    if (!strcasecmp(word1, "boyfriend") || !strcasecmp(word1, "bf")) {
       strlcpy(whotarget, getxtra(hand, "BF"), sizeof whotarget);
       if (whotarget[0]) {
         snprintf(whoredirect, sizeof whoredirect, "%s, your boyfriend is %s, and ", nick, whotarget);
@@ -272,8 +272,8 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand,
       }
     }
     /* "my girlfriend" */
-    else if (!egg_strcasecmp(word1, "girlfriend") ||
-             !egg_strcasecmp(word1, "gf")) {
+    else if (!strcasecmp(word1, "girlfriend") ||
+             !strcasecmp(word1, "gf")) {
       strlcpy(whotarget, getxtra(hand, "GF"), sizeof whotarget);
       if (whotarget[0]) {
         snprintf(whoredirect, sizeof whoredirect, "%s, your girlfriend is %s, and ",
@@ -290,19 +290,19 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand,
     }
   }
   /* "your" keyword */
-  else if (!egg_strcasecmp(word1, "your")) {
+  else if (!strcasecmp(word1, "your")) {
     wordshift(word1, text);
     /* "your admin" */
-    if (!egg_strcasecmp(word1, "owner") || !egg_strcasecmp(word1, "admin")) {
+    if (!strcasecmp(word1, "owner") || !strcasecmp(word1, "admin")) {
       if (admin[0]) {
         strlcpy(word2, admin, sizeof word2);
         wordshift(whotarget, word2);
         strcat(whoredirect, "My owner is ");
         strcat(whoredirect, whotarget);
         strcat(whoredirect, ", and ");
-        if (!egg_strcasecmp(whotarget, hand)) {
+        if (!strcasecmp(whotarget, hand)) {
           strcat(whoredirect, "that's YOU");
-          if (!egg_strcasecmp(hand, nick))
+          if (!strcasecmp(hand, nick))
             strcat(whoredirect, "!!!");
           else {
             strcat(whoredirect, ", ");
@@ -348,7 +348,7 @@ targetcont:
       onchan = 1;
       snprintf(word1, sizeof word1, "%s!%s", whotarget, m->userhost);
       urec = get_user_by_host(word1);
-      if (!urec || !egg_strcasecmp(whotarget, urec->handle))
+      if (!urec || !strcasecmp(whotarget, urec->handle))
         break;
       strcat(whoredirect, whotarget);
       strcat(whoredirect, " is ");
@@ -366,8 +366,7 @@ targetcont:
       while (m && m->nick[0]) {
         snprintf(word2, sizeof word2, "%s!%s", m->nick, m->userhost);
         urec = get_user_by_host(word2);
-        if (urec && !egg_strcasecmp(urec->handle, whotarget)) {
-          onchan = 1;
+        if (urec && !strcasecmp(urec->handle, whotarget)) {
           strcat(whoredirect, whotarget);
           strcat(whoredirect, " is ");
           strcat(whoredirect, m->nick);
@@ -427,7 +426,7 @@ targetcont:
   /* Is the target currently DCC CHAT to me on the botnet? */
   for (i = 0; i < dcc_total; i++) {
     if (dcc[i].type->flags & DCT_CHAT) {
-      if (!egg_strcasecmp(whotarget, dcc[i].nick)) {
+      if (!strcasecmp(whotarget, dcc[i].nick)) {
         if (!rfc_casecmp(channel, dcc[i].u.chat->con_chan) &&
             dcc[i].u.chat->con_flags & LOG_PUBLIC) {
           strcat(whoredirect, whotarget);
@@ -445,7 +444,7 @@ targetcont:
   }
   /* Target known, but nowhere to be seen.  Give last IRC and botnet time */
   wordshift(word1, text);
-  if (!egg_strcasecmp(word1, "anywhere"))
+  if (!strcasecmp(word1, "anywhere"))
     cr = NULL;
   else
     for (cr = urec->chanrec; cr; cr = cr->next) {
@@ -472,23 +471,23 @@ targetcont:
   work = now - laston;
   if (work >= 86400) {
     tv = work / 86400;
-    snprintf(word2, sizeof word2, "%lu day%s, ", tv, (tv == 1) ? "" : "s");
+    snprintf(word2, sizeof word2, "%li day%s, ", tv, (tv == 1) ? "" : "s");
     work = work % 86400;
   }
   if (work >= 3600) {
     tv = work / 3600;
-    sprintf(word2 + strlen(word2), "%lu hour%s, ", tv, (tv == 1) ? "" : "s");
+    snprintf(word2 + strlen(word2), (sizeof word2) - strlen(word2), "%li hour%s, ", tv, (tv == 1) ? "" : "s");
     work = work % 3600;
   }
   if (work >= 60) {
     tv = work / 60;
-    sprintf(word2 + strlen(word2), "%lu minute%s, ", tv,
+    snprintf(word2 + strlen(word2), (sizeof word2) - strlen(word2), "%li minute%s, ", tv,
             (tv == 1) ? "" : "s");
   }
   if (!word2[0] && (work < 60)) {
     strlcpy(word2, "just moments ago!!", sizeof word2);
   } else {
-    strcpy(word2 + strlen(word2) - 2, " ago.");
+    strlcpy(word2 + strlen(word2) - 2, " ago.", (sizeof word2) - strlen(word2) + 2);
   }
   if (lastonplace[0] && (strchr(CHANMETA, lastonplace[0]) != NULL))
     snprintf(word1, sizeof word1, "on IRC channel %s", lastonplace);
@@ -531,7 +530,7 @@ static char *match_trigger(char *word)
   trig_data *t = trigdata;
 
   while (t->key) {
-    if (!egg_strcasecmp(word, t->key))
+    if (!strcasecmp(word, t->key))
       return t->text;
     t++;
   }
@@ -550,7 +549,7 @@ static char *getxtra(char *hand, char *field)
     ue = find_user_entry(&USERENTRY_XTRA, urec);
     if (ue)
       for (xk = ue->u.extra; xk; xk = xk->next)
-        if (xk->key && !egg_strcasecmp(xk->key, field)) {
+        if (xk->key && !strcasecmp(xk->key, field)) {
           if (xk->data[0] == '{' && xk->data[strlen(xk->data) - 1] == '}' &&
               strlen(xk->data) > 2) {
             strncpy(fixit, &xk->data[1], strlen(xk->data) - 2);
@@ -572,7 +571,7 @@ static void wordshift(char *first, char *rest)
     p = newsplit(&q);
     strcpy(first, p);
     memmove(rest, q, strlen(q) + 1);
-  } while (!egg_strcasecmp(first, "and") || !egg_strcasecmp(first, "or"));
+  } while (!strcasecmp(first, "and") || !strcasecmp(first, "or"));
 }
 
 /* Report on current seen info for .modulestat. */
