@@ -97,7 +97,7 @@ static int msgrate;             /* Number of seconds between sending
                                  * queued lines to server. */
 #ifdef TLS
 static int use_ssl;             /* Use SSL for the next server connection? */
-static int tls_vfyserver;       /* Certificate validation mode for servrs  */
+static int tls_vfyserver;       /* Certificate validation mode for servers */
 #endif
 
 #ifndef TLS
@@ -105,7 +105,7 @@ static char sslserver = 0;
 #endif
 
 static p_tcl_bind_list H_wall, H_raw, H_notc, H_msgm, H_msg, H_flud, H_ctcr,
-                       H_ctcp, H_out, H_rawt, H_account;
+                       H_ctcp, H_out, H_rawt;
 
 static void empty_msgq(void);
 static void next_server(int *, char *, unsigned int *, char *);
@@ -1279,17 +1279,6 @@ static int server_msg STDVAR
   return TCL_OK;
 }
 
-static int server_account STDVAR
-{
-  Function F = (Function) cd;
-
-  BADARGS(6, 6, " nick uhost hand chan account");
-
-  CHECKVALIDITY(server_account);
-  F(argv[1], argv[2], get_user_by_handle(userlist, argv[3]), argv[4], argv[5]);
-  return TCL_OK;
-}
-
 static int server_raw STDVAR
 {
   Function F = (Function) cd;
@@ -1301,14 +1290,24 @@ static int server_raw STDVAR
   return TCL_OK;
 }
 
-static int server_tag STDVAR
+static int server_rawt STDVAR
 {
+  int unused;
+  Tcl_Obj *tagdict;
   Function F = (Function) cd;
 
-  BADARGS(5, 5, " from code args tag");
+  BADARGS(5, 5, " from code args tagdict");
 
-  CHECKVALIDITY(server_tag);
-  Tcl_AppendResult(irp, int_to_base10(F(argv[1], argv[3], argv[4])), NULL);
+  CHECKVALIDITY(server_rawt);
+  tagdict = Tcl_NewStringObj(argv[4], -1);
+  if (Tcl_DictObjSize(irp, tagdict, &unused) != TCL_OK) {
+    /* check early, Tcl sets error string first */
+    Tcl_AppendResult(irp, " in call to ", argv[0], NULL);
+    return TCL_ERROR;
+  }
+  Tcl_IncrRefCount(tagdict);
+  Tcl_AppendResult(irp, int_to_base10(F(argv[1], argv[3], tagdict)), NULL);
+  Tcl_DecrRefCount(tagdict);
   return TCL_OK;
 }
 
@@ -1662,6 +1661,7 @@ static tcl_ints my_tcl_ints[] = {
   {"message-tags",      &message_tags,              0},
   {"extended-join",     &extended_join,             0},
   {"account-notify",    &account_notify,            0},
+  {"account-tag",       &account_tag,               0},
   {NULL,                NULL,                       0}
 };
 
@@ -1857,7 +1857,7 @@ static void dcc_chat_hostresolved(int i)
 #ifdef TLS
   else if (dcc[i].ssl && ssl_handshake(dcc[i].sock, TLS_CONNECT, tls_vfydcc,
                                        LOG_MISC, dcc[i].host, &dcc_chat_sslcb))
-    egg_snprintf(buf, sizeof buf, "TLS negotiation error");
+    strlcpy(buf, "TLS negotiation error", sizeof buf);
 #endif
   if (buf[0]) {
     if (!quiet_reject)
@@ -1916,7 +1916,7 @@ static void server_5minutely()
 
       disconnect_server(servidx);
       lostdcc(servidx);
-      putlog(LOG_SERV, "*", IRC_SERVERSTONED);
+      putlog(LOG_SERV, "*", "%s", IRC_SERVERSTONED);
     } else if (!trying_server) {
       /* Check for server being stoned. */
       dprintf(DP_MODE, "PING :%li\n", now);
@@ -2036,7 +2036,7 @@ static void server_report(int idx, int details)
     egg_snprintf(s, sizeof s, "(connected %s)", s1);
     if (server_lag && !lastpingcheck) {
       if (server_lag == -1)
-        egg_snprintf(s1, sizeof s1, " (bad pong replies)");
+        strlcpy(s1, " (bad pong replies)", sizeof s1);
       else
         egg_snprintf(s1, sizeof s1, " (lag: %ds)", server_lag);
       strcat(s, s1);
@@ -2129,7 +2129,6 @@ static char *server_close()
   isupport_fini();
   /* Restore original commands. */
   del_bind_table(H_wall);
-  del_bind_table(H_account);
   del_bind_table(H_raw);
   del_bind_table(H_rawt);
   del_bind_table(H_notc);
@@ -2236,7 +2235,7 @@ static Function server_table[] = {
   /* 40 - 43 */
   (Function) & H_out,           /* p_tcl_bind_list                      */
   (Function) & net_type_int,    /* int                                  */
-  (Function) & H_account,       /* p_tcl_bind)list                      */
+  (Function) NULL,              /* was H_account, now irc.mod           */
   (Function) & cap,             /* capability_t                         */
   /* 44 - 47 */
   (Function) & extended_join,   /* int                                  */
@@ -2244,7 +2243,10 @@ static Function server_table[] = {
   (Function) & H_isupport,      /* p_tcl_bind_list                      */
   (Function) & isupport_get,    /*                                      */
   /* 48 - 52 */
-  (Function) & isupport_parseint/*                                      */
+  (Function) & isupport_parseint,/*                                     */
+  (Function) NULL,               /* was check_tcl_account, now irc.mod  */
+  (Function) & find_capability,
+  (Function) encode_msgtags
 };
 
 char *server_start(Function *global_funcs)
@@ -2345,9 +2347,8 @@ char *server_start(Function *global_funcs)
                TCL_TRACE_READS | TCL_TRACE_WRITES | TCL_TRACE_UNSETS,
                traced_nicklen, NULL);
   H_wall = add_bind_table("wall", HT_STACKABLE, server_2char);
-  H_account = add_bind_table("account", HT_STACKABLE, server_account);
   H_raw = add_bind_table("raw", HT_STACKABLE, server_raw);
-  H_rawt = add_bind_table("rawt", HT_STACKABLE, server_tag);
+  H_rawt = add_bind_table("rawt", HT_STACKABLE, server_rawt);
   H_notc = add_bind_table("notc", HT_STACKABLE, server_5char);
   H_msgm = add_bind_table("msgm", HT_STACKABLE, server_msg);
   H_msg = add_bind_table("msg", 0, server_msg);
