@@ -31,13 +31,16 @@
 #include "../channels.mod/channels.h"
 #include "server.h"
 
+char *encode_msgtags(Tcl_Obj *msgtagdict);
+static char *encode_msgtag(char *key, char *value);
 static int del_capabilities(char *);
 static int del_capability(char *name);
 static time_t last_ctcp = (time_t) 0L;
 static int multistatus = 0, count_ctcp = 0;
 static char altnick_char = 0;
 struct capability *cap;
-int account_notify, extended_join;
+struct capability *find_capability(char *capname);
+int account_notify = 1, extended_join = 1, account_tag = 0;
 Tcl_Obj *ncapeslist;
 
 /* We try to change to a preferred unique nick here. We always first try the
@@ -133,7 +136,7 @@ static int check_tcl_msgm(char *cmd, char *nick, char *uhost,
   char args[1024];
 
   if (arg[0])
-    simple_sprintf(args, "%s %s", cmd, arg);
+    snprintf(args, sizeof args, "%s %s", cmd, arg);
   else
     strlcpy(args, cmd, sizeof args);
   get_user_flagrec(u, &fr, NULL);
@@ -193,35 +196,23 @@ static int check_tcl_raw(char *from, char *code, char *msg)
   Tcl_SetVar(interp, "_raw2", code, 0);
   Tcl_SetVar(interp, "_raw3", msg, 0);
   x = check_tcl_bind(H_raw, code, 0, " $_raw1 $_raw2 $_raw3",
-                     MATCH_EXACT | BIND_STACKABLE | BIND_WANTRET);
+                     MATCH_MASK | BIND_STACKABLE | BIND_WANTRET);
 
   /* Return 1 if processed */
   return (x == BIND_EXEC_LOG);
 }
 
-/* tagstr is a space-separated list of key/value pairs */
-static int check_tcl_rawt(char *from, char *code, char *msg, char *tagstr)
+/* tagstr is the string value of a Tcl dictionary (flat key/value list) */
+static int check_tcl_rawt(char *from, char *code, char *msg, char *tagdict)
 {
   int x;
-  char * ptr;
-  Tcl_DString tagdict;
 
-  Tcl_DStringInit(&tagdict);
   Tcl_SetVar(interp, "_rawt1", from, 0);
   Tcl_SetVar(interp, "_rawt2", code, 0);
   Tcl_SetVar(interp, "_rawt3", msg, 0);
-  ptr = strtok(tagstr, " ");
-  Tcl_DStringAppendElement(&tagdict, ptr);
-  while (ptr != NULL) {
-    ptr = strtok(NULL, " ");
-    if (ptr) {
-      Tcl_DStringAppendElement(&tagdict, ptr);
-    }
-  }
-  Tcl_SetVar(interp, "_rawt4", Tcl_DStringValue(&tagdict), 0);
+  Tcl_SetVar(interp, "_rawt4", tagdict, 0);
   x = check_tcl_bind(H_rawt, code, 0, " $_rawt1 $_rawt2 $_rawt3 $_rawt4",
-                    MATCH_EXACT | BIND_STACKABLE | BIND_WANTRET);
-  Tcl_DStringFree(&tagdict);
+                    MATCH_MASK | BIND_STACKABLE | BIND_WANTRET);
   return (x == BIND_EXEC_LOG);
 }
 
@@ -268,22 +259,6 @@ static int check_tcl_wall(char *from, char *msg)
   return 1;
 }
 
-static int check_tcl_account(char *nick, char *uhost, char *mask,
-                            struct userrec *u, char *chan,  char *account)
-{
-  struct flag_record fr = { FR_GLOBAL | FR_CHAN | FR_ANYWH, 0, 0, 0, 0, 0 };
-  int x;
-
-  Tcl_SetVar(interp, "_acnt1", nick, 0);
-  Tcl_SetVar(interp, "_acnt2", uhost, 0);
-  Tcl_SetVar(interp, "_acnt3", u ? u->handle : "*", 0);
-  Tcl_SetVar(interp, "_acnt4", chan, 0);
-  Tcl_SetVar(interp, "_acnt5", account, 0);
-  x = check_tcl_bind(H_account, mask, &fr,
-       " $_acnt1 $_acnt2 $_acnt3 $_acnt4 $_acnt5", MATCH_MASK | BIND_STACKABLE);
-  return (x == BIND_EXEC_LOG);
-}
-
 static int check_tcl_flud(char *nick, char *uhost, struct userrec *u,
                           char *ftype, char *chname)
 {
@@ -336,6 +311,29 @@ static int match_my_nick(char *nick)
   return (!rfc_casecmp(nick, botname));
 }
 
+char *encode_msgtags(Tcl_Obj *msgtagdict) {
+  int done = 0;
+  Tcl_DictSearch s;
+  Tcl_Obj *value, *key;
+  static Tcl_DString ds;
+  static int ds_initialized = 0;
+
+  if (!ds_initialized) {
+    Tcl_DStringInit(&ds);
+    ds_initialized = 1;
+  } else {
+    Tcl_DStringFree(&ds);
+  }
+  for (Tcl_DictObjFirst(interp, msgtagdict, &s, &key, &value, &done); !done; Tcl_DictObjNext(&s, &key, &value, &done)) {
+    if (Tcl_DStringLength(&ds)) {
+      Tcl_DStringAppend(&ds, ";", -1);
+    }
+    Tcl_DStringAppend(&ds, encode_msgtag(Tcl_GetString(key), Tcl_GetString(value)), -1);
+  }
+
+  return Tcl_DStringValue(&ds);
+}
+
 /* 001: welcome to IRC (use it to fix the server name) */
 static int got001(char *from, char *msg)
 {
@@ -529,7 +527,7 @@ static int detect_flood(char *floodnick, char *floodhost, char *from, int which)
     if (check_tcl_flud(floodnick, floodhost, u, ftype, "*"))
       return 0;
     /* Private msg */
-    simple_sprintf(h, "*!*@%s", p);
+    snprintf(h, sizeof h, "*!*@%s", p);
     putlog(LOG_MISC, "*", IRC_FLOODIGNORE1, p);
     addignore(h, botnetnick, (which == FLOOD_CTCP) ? "CTCP flood" :
               "MSG/NOTICE flood", now + (60 * ignore_time));
@@ -537,9 +535,10 @@ static int detect_flood(char *floodnick, char *floodhost, char *from, int which)
   return 0;
 }
 
+
 /* Got a private message.
  */
-static int gotmsg(char *from, char *msg, char *tag)
+static int gotmsg(char *from, char *msg)
 {
   char *to, buf[UHOSTLEN], *nick, ctcpbuf[512], *uhost = buf, *ctcp,
        *p, *p1, *code;
@@ -574,7 +573,7 @@ static int gotmsg(char *from, char *msg, char *tag)
       ctcp = ctcpbuf;
 
       /* remove the ctcp in msg */
-      memmove(p1 - 1, p + 1, strlen(p + 1) + 1);
+      memmove(p1 - 1, p + 1, strlen(p));
 
       if (!ignoring)
         detect_flood(nick, uhost, from,
@@ -644,7 +643,6 @@ static int gotmsg(char *from, char *msg, char *tag)
   }
   if (msg[0]) {
     int result = 0;
-
     /* Msg from oper, don't interpret */
     if ((to[0] == '$') || (strchr(to, '.') != NULL)) {
       if (!ignoring) {
@@ -764,14 +762,17 @@ static int gotnotice(char *from, char *msg)
   return 0;
 }
 
-static int gottagmsg(char *from, char *msg) {
-  char *nick;
+static int gottagmsg(char *from, char *msg, Tcl_Obj *tagdict) {
+  char *nick, *dictstring;
+
+  dictstring = encode_msgtags(tagdict);
+
   fixcolon(msg);
   if (strchr(from, '!')) {
     nick = splitnick(&from);
-    putlog(LOG_SERV, "*", "[#]%s(%s)[#] %s", nick, from, msg);
+    putlog(LOG_SERV, "*", "[#]%s(%s)[#] TAGMSG: %s", nick, from, dictstring);
   } else {
-    putlog(LOG_SERV, "*", "[#]%s[#] %s");
+    putlog(LOG_SERV, "*", "[#]%s[#] TAGMSG: %s", from, dictstring);
   }
   return 0;
 }
@@ -876,7 +877,7 @@ static int got432(char *from, char *msg)
     putlog(LOG_MISC, "*", "NICK IS INVALID: '%s' (keeping '%s').", erroneous,
            botname);
   else {
-    putlog(LOG_MISC, "*", IRC_BADBOTNICK);
+    putlog(LOG_MISC, "*", "%s", IRC_BADBOTNICK);
     if (!strcmp(erroneous, origbotname)) {
       strlcpy(nick, get_altbotnick(), sizeof nick);
     } else {
@@ -1129,15 +1130,82 @@ static struct dcc_table SERVER_SOCKET = {
   display_server,
   NULL,
   kill_server,
+  NULL,
   NULL
 };
 
+static char *encode_msgtag_value(char *value)
+{
+  static char buf[TOTALTAGMAX+1];
+  size_t written = 0;
+
+  /* empty value and no value is identical, Tcl dict always has empty string as no-value, looks better to encode without = */
+  if (!value || !*value) {
+    return "";
+  }
+  buf[written++] = '=';
+
+  while (*value && written < sizeof buf - 1) {
+    if (*value == ';' || *value == ' ' || *value == '\\' || *value == '\r' || *value == '\n') {
+      buf[written++] = '\\';
+    }
+    buf[written++] = *value++;
+  }
+  buf[written] = '\0';
+
+  return buf;
+}
+
+/* TODO: validity enforcement on used characters in key? */
+static char *encode_msgtag(char *key, char *value)
+{
+  static char buf[TOTALTAGMAX+1];
+
+  snprintf(buf, sizeof buf, "%s%s", key, encode_msgtag_value(value));
+  return buf;
+}
+
+static char *decode_msgtag_value(char *value, char **endptr)
+{
+  static char valuebuf[TOTALTAGMAX+1];
+  char *tmp, *decoded = valuebuf;
+  int escaped = 0;
+
+  for (tmp = value; *tmp && *tmp != ';' && *tmp != ' ' && decoded - valuebuf < TOTALTAGMAX; tmp++) {
+    if (!escaped && *tmp == '\\') {
+      escaped = 1;
+      continue;
+    }
+    if (escaped) {
+      if (*tmp == ':') {
+        *decoded++ = ';';
+      } else if (*tmp == 'n') {
+        *decoded++ = '\n';
+      } else if (*tmp == 'r') {
+        *decoded++ = '\r';
+      } else if (*tmp == 's') {
+        *decoded++ = ' ';
+      } else {
+        *decoded++ = *tmp;
+      }
+      escaped = 0;
+    } else {
+      *decoded++ = *tmp;
+    }
+  }
+  /* either points to \0 or ; */
+  *endptr = tmp;
+
+  *decoded = '\0';
+  return valuebuf;
+}
+
 static void server_activity(int idx, char *tagmsg, int len)
 {
-  char *from, *code, *s1, *s2, *saveptr1=NULL, *saveptr2=NULL, *tagstrptr=NULL;
-  char *token, *subtoken, tagstr[TOTALTAGMAX+1], tagdict[TOTALTAGMAX+1] = "";
-  char *msgptr, rawmsg[RECVLINEMAX+7];
-  int taglen, i, found, ret;
+  char *from, *code, *msgptr;
+  char rawmsg[RECVLINEMAX+7];
+  int ret;
+  Tcl_Obj *tagdict = Tcl_NewDictObj();
 
   if (trying_server) {
     strcpy(dcc[idx].nick, "(server)");
@@ -1146,44 +1214,25 @@ static void server_activity(int idx, char *tagmsg, int len)
     SERVER_SOCKET.timeout_val = 0;
   }
   lastpingcheck = 0;
-/* Check if message-tags are enabled and, if so, check/grab the tag */
+  /* Parse optional message-tags, regardless of whether they are enabled on our side */
   msgptr = tagmsg;
   strlcpy(rawmsg, tagmsg, TOTALTAGMAX+1);
   if (*tagmsg == '@') {
-    taglen = 0;
-    tagstrptr = newsplit(&msgptr);
-    strlcpy(tagstr, tagstrptr, TOTALTAGMAX+1);
-    tagstrptr++;     /* Remove @ */
-    /* Split each key/value pair apart, then split the key from the value */
-    for (i = 0, s1 = tagstrptr; ; i++, s1 = NULL){
-      token = strtok_r(s1, ";", &saveptr1);
-      if (token == NULL) {
-        break;
+    char *key, *value, *lastendptr = tagmsg;
+
+    do {
+      key = lastendptr + 1;
+      value = key + strcspn(key, "=; ");
+
+      if (*value == '=') {
+        Tcl_DictObjPut(interp, tagdict, Tcl_NewStringObj(key, value - key), Tcl_NewStringObj(decode_msgtag_value(value + 1, &lastendptr), -1));
+      } else {
+        Tcl_DictObjPut(interp, tagdict, Tcl_NewStringObj(key, value - key), Tcl_NewStringObj("", -1));
+        lastendptr = value;
       }
-      if (*token == '+') {
-        token++;
-      }
-      if (strchr(token, '=')) {
-        found = 0;
-        for (s2 = token; ; s2 = NULL) {
-          subtoken = strtok_r(s2, "=", &saveptr2);
-          if (subtoken == NULL) {
-            break;
-          }
-          taglen += egg_snprintf(tagdict + taglen, TOTALTAGMAX - taglen,
-                "%s ", subtoken);
-          found++;
-        }
-        /* Account for tags (not key/value pairs), prep empty value for Tcl */
-        if (found < 2) {
-          taglen += egg_snprintf(tagdict + taglen, TOTALTAGMAX - taglen,
-              "{} ");
-        }
-      }
-    }
-    if (taglen > 0) {
-      tagdict[taglen-1] = '\0';     /* Remove trailing space */
-    }
+    } while (*lastendptr && *lastendptr != ' ');
+
+    msgptr = lastendptr + (*lastendptr != '\0');
   }
   from = "";
   if (*msgptr == ':') {
@@ -1197,7 +1246,9 @@ static void server_activity(int idx, char *tagmsg, int len)
   }
   /* Check both raw and rawt, to allow backwards compatibility with older
    * scripts. If rawt returns 1 (blocking), don't process raw binds.*/
-  ret = check_tcl_rawt(from, code, msgptr, tagdict);
+  /* Tcl_GetString() must not be modified, so we have to copy because string C API is not const char* */
+  strlcpy(rawmsg, Tcl_GetString(tagdict), sizeof rawmsg);
+  ret = check_tcl_rawt(from, code, msgptr, rawmsg);
   if (!ret) {
     check_tcl_raw(from, code, msgptr);
   }
@@ -1373,23 +1424,24 @@ static int tryauthenticate(char *from, char *msg)
       }
     }
 #else
-      putlog(LOG_DEBUG, "*", "SASL: TLS libs missing EC support, try PLAIN or "
-                "EXTERNAL method");
+      putlog(LOG_DEBUG, "*", "SASL: TLS libs not present or missing EC support."
+                " Try the PLAIN or EXTERNAL method instead");
       return 1;
     }
 #endif
     else {          /* sasl_mechanism == SASL_MECHANISM_EXTERNAL */
-#ifdef TLS          /* TLS required for EXTERNAL sasl */ 
+#ifdef TLS          /* TLS required for EXTERNAL sasl */
       dst[0] = '+';
       dst[1] = 0;
     }
-    putlog(LOG_DEBUG, "*", "SASL: put AUTHENTICATE %s", dst);
-    dprintf(DP_MODE, "AUTHENTICATE %s\n", dst);
 #else
-    putlog(LOG_DEBUG, "*", "SASL: TLS libs required for EXTERNAL but are not "
+      putlog(LOG_DEBUG, "*", "SASL: TLS libs required for EXTERNAL but are not "
             "installed, try PLAIN method");
+      return 1;
     }
 #endif /* TLS */
+    putlog(LOG_DEBUG, "*", "SASL: put AUTHENTICATE %s", dst);
+    dprintf(DP_MODE, "AUTHENTICATE %s\n", dst);
   } else {      /* Only EC-challenges get extra auth messages w/o a + */
 #ifdef TLS
 #ifdef HAVE_EVP_PKEY_GET1_EC_KEY
@@ -1578,34 +1630,6 @@ static int handle_sasl_timeout()
   return sasl_error("timeout");
 }
 
-/* Got ACCOUNT message; only valid for account-notify capability */
-static int gotaccount(char *from, char *msg) {
-  struct chanset_t *chan;
-  struct userrec *u;
-  memberlist *m;
-  char *nick, *chname, mask[CHANNELLEN+UHOSTLEN+NICKMAX+2];
-
-  u = get_user_by_host(from);
-  nick = splitnick(&from);
-  for (chan = chanset; chan; chan = chan->next) {
-    chname = chan->dname;
-    if ((m = ismember(chan, nick))) {
-      strlcpy (m->account, msg, sizeof m->account);
-      snprintf(mask, sizeof mask, "%s %s", chname, from);
-      if (!strcasecmp(msg, "*")) {
-        msg[0] = '\0';
-        putlog(LOG_JOIN | LOG_MISC, chname, "%s!%s has logged out of their "
-                "account", nick, from);
-      } else {
-        putlog(LOG_JOIN | LOG_MISC, chname, "%s!%s has logged into account %s",
-                nick, from, msg);
-      }
-      check_tcl_account(nick, from, mask, u, chname, msg);
-    }
-  }
-  return 0;
-}
-
 /*
  * 465     ERR_YOUREBANNEDCREEP :You are banned from this server
  */
@@ -1819,6 +1843,9 @@ static int gotcap(char *from, char *msg) {
       } else if (!strcmp(current->name, "account-notify") && (account_notify)
                 && (!current->enabled)) {
         add_req(current->name);
+      } else if (!strcmp(current->name, "account-tag") && (account_tag)
+                && (!current->enabled)) {
+        add_req(current->name);
       } else if (!strcmp(current->name, "extended-join") && (extended_join) 
                 && (!current->enabled)) {
         add_req(current->name);
@@ -2017,7 +2044,6 @@ static cmd_t my_raw_binds[] = {
   {"KICK",         "",   (IntFunc) gotkick,         NULL},
   {"CAP",          "",   (IntFunc) gotcap,          NULL},
   {"AUTHENTICATE", "",   (IntFunc) gotauthenticate, NULL},
-  {"ACCOUNT",      "",   (IntFunc) gotaccount,      NULL},
   {"CHGHOST",      "",   (IntFunc) gotchghost,      NULL},
   {"SETNAME",      "",   (IntFunc) gotsetname,      NULL},
   {NULL,           NULL, NULL,                      NULL}
