@@ -73,7 +73,7 @@ tandbuf *tbuf;
 
 /* Prototypes */
 static void start_sending_users(int);
-static void shareout_but EGG_VARARGS(struct chanset_t *, arg1);
+static void shareout_but(struct chanset_t *chan, int x, const char *format, ...) ATTRIBUTE_FORMAT(printf,3,4);
 static int flush_tbuf(char *);
 static int can_resync(char *);
 static void dump_resync(int);
@@ -568,6 +568,22 @@ static void share_newchan(int idx, char *par)
   }
 }
 
+static void share_pls_account(int idx, char *par)
+{
+  char *hand;
+  struct userrec *u;
+
+  if ((dcc[idx].status & STAT_SHARE) && !private_user) {
+    hand = newsplit(&par);
+    if ((u = get_user_by_handle(userlist, hand)) &&
+        !(u->flags & USER_UNSHARED)) {
+      shareout_but(NULL, idx, "+a %s %s\n", hand, par);
+      set_user(&USERENTRY_ACCOUNT, u, par);
+      putlog(LOG_CMDS, "*", "%s: +account %s %s", dcc[idx].nick, hand, par);
+    }
+  }
+}
+
 static void share_pls_host(int idx, char *par)
 {
   char *hand;
@@ -584,6 +600,33 @@ static void share_pls_host(int idx, char *par)
   }
 }
 
+static void share_pls_botaccount(int idx, char *par)
+{
+  char *hand, pass[PASSWORDLEN];
+  struct userrec *u;
+
+  if ((dcc[idx].status & STAT_SHARE) && !private_user) {
+    hand = newsplit(&par);
+    if (!(u = get_user_by_handle(userlist, hand)) ||
+        !(u->flags & USER_UNSHARED)) {
+      if (!(dcc[idx].status & STAT_GETTING))
+        shareout_but(NULL, idx, "+ba %s %s\n", hand, par);
+      /* Add bot to userlist if not there */
+      if (u) {
+        if (!(u->flags & USER_BOT))
+          return;               /* ignore */
+        set_user(&USERENTRY_ACCOUNT, u, par);
+      } else {
+        makepass(pass);
+        userlist = adduser(userlist, hand, par, pass, USER_BOT);
+        explicit_bzero(pass, sizeof pass);
+      }
+      if (!(dcc[idx].status & STAT_GETTING))
+        putlog(LOG_CMDS, "*", "%s: +account %s %s", dcc[idx].nick, hand, par);
+    }
+  }
+}
+
 static void share_pls_bothost(int idx, char *par)
 {
   char *hand, pass[PASSWORDLEN];
@@ -611,6 +654,24 @@ static void share_pls_bothost(int idx, char *par)
   }
 }
 
+static void share_mns_account(int idx, char *par)
+{
+  char *hand;
+  struct userrec *u;
+
+  if ((dcc[idx].status & STAT_SHARE) && !private_user) {
+    hand = newsplit(&par);
+    if ((u = get_user_by_handle(userlist, hand)) &&
+        !(u->flags & USER_UNSHARED)) {
+      shareout_but(NULL, idx, "-a %s %s\n", hand, par);
+      noshare = 1;
+      delaccount_by_handle(hand, par);
+      noshare = 0;
+      putlog(LOG_CMDS, "*", "%s: -account %s %s", dcc[idx].nick, hand, par);
+    }
+  }
+}
+
 static void share_mns_host(int idx, char *par)
 {
   char *hand;
@@ -1328,7 +1389,9 @@ static void share_feats(int idx, char *par)
  */
 static botscmd_t C_share[] = {
   {"!",        "",  (IntFunc) share_endstartup},
+  {"+a",       "psu", (IntFunc) share_pls_account},
   {"+b",       "psb", (IntFunc) share_pls_ban},
+  {"+ba",      "psb", (IntFunc) share_pls_botaccount},
   {"+bc",      "psb", (IntFunc) share_pls_banchan},
   {"+bh",      "psu", (IntFunc) share_pls_bothost},
   {"+cr",      "psc", (IntFunc) share_pls_chrec},
@@ -1338,6 +1401,7 @@ static botscmd_t C_share[] = {
   {"+i",       "psn", (IntFunc) share_pls_ignore},
   {"+inv",     "psj", (IntFunc) share_pls_invite},
   {"+invc",    "psj", (IntFunc) share_pls_invitechan},
+  {"-a",       "psu", (IntFunc) share_mns_account},
   {"-b",       "psb", (IntFunc) share_mns_ban},
   {"-bc",      "psb", (IntFunc) share_mns_banchan},
   {"-cr",      "psc", (IntFunc) share_mns_chrec},
@@ -1398,18 +1462,15 @@ static void sharein_mod(int idx, char *msg)
   }
 }
 
-static void shareout_mod EGG_VARARGS_DEF(struct chanset_t *, arg1)
+ATTRIBUTE_FORMAT(printf,2,3)
+static void shareout_mod(struct chanset_t *chan, const char *format, ...)
 {
   int i, l;
-  char *format;
   char s[601];
-  struct chanset_t *chan;
   va_list va;
 
-  chan = EGG_VARARGS_START(struct chanset_t *, arg1, va);
-
   if (!chan || channel_shared(chan)) {
-    format = va_arg(va, char *);
+    va_start(va, format);
 
     strcpy(s, "s ");
     if ((l = egg_vsnprintf(s + 2, 509, format, va)) < 0)
@@ -1428,21 +1489,18 @@ static void shareout_mod EGG_VARARGS_DEF(struct chanset_t *, arg1)
         }
       }
     q_resync(s, chan);
+    va_end(va);
   }
-  va_end(va);
 }
 
-static void shareout_but EGG_VARARGS_DEF(struct chanset_t *, arg1)
+ATTRIBUTE_FORMAT(printf,3,4)
+static void shareout_but(struct chanset_t *chan, int x, const char *format, ...)
 {
-  int i, x, l;
-  char *format;
+  int i, l;
   char s[601];
-  struct chanset_t *chan;
   va_list va;
 
-  chan = EGG_VARARGS_START(struct chanset_t *, arg1, va);
-  x = va_arg(va, int);
-  format = va_arg(va, char *);
+  va_start(va, format);
 
   strcpy(s, "s ");
   if ((l = egg_vsnprintf(s + 2, 509, format, va)) < 0)
@@ -1716,7 +1774,7 @@ static int write_tmp_userfile(char *fn, struct userrec *bu, int idx)
     fclose(f);
   }
   if (!ok)
-    putlog(LOG_MISC, "*", USERF_ERRWRITE2);
+    putlog(LOG_MISC, "*", "%s", USERF_ERRWRITE2);
   return ok;
 }
 
@@ -1978,11 +2036,7 @@ static void start_sending_users(int idx)
   int i = 1;
   struct chanuserrec *ch;
   struct chanset_t *cst;
-#ifdef IPV6
-  char s[INET6_ADDRSTRLEN];
-#else
-  char s[INET_ADDRSTRLEN];
-#endif
+  char s[EGG_INET_ADDRSTRLEN];
 
   egg_snprintf(share_file, sizeof share_file, ".share.%s.%lu", dcc[idx].nick,
                now);
@@ -2142,12 +2196,12 @@ static void cancel_user_xfer(int idx, void *x)
 }
 
 static tcl_ints my_ints[] = {
-  {"allow-resync",      &allow_resync},
-  {"resync-time",        &resync_time},
-  {"private-global",  &private_global},
-  {"private-user",      &private_user},
-  {"override-bots", &overr_local_bots},
-  {NULL,                         NULL}
+  {"allow-resync",      &allow_resync, 0},
+  {"resync-time",        &resync_time, 0},
+  {"private-global",  &private_global, 0},
+  {"private-user",      &private_user, 0},
+  {"override-bots", &overr_local_bots, 0},
+  {NULL,                         NULL, 0}
 };
 
 static tcl_strings my_strings[] = {
