@@ -1,9 +1,7 @@
 /*
  * tcltransfer.c -- part of transfer.mod
  *
- * $Id: tcltransfer.c,v 1.11 2011/02/13 14:19:34 simple Exp $
- *
- * Copyright (C) 2003 - 2011 Eggheads Development Team
+ * Copyright (C) 2003 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -52,24 +50,9 @@ static int tcl_dccsend STDVAR
     Tcl_AppendResult(irp, "4", NULL);
     return TCL_OK;
   }
-  if (copy_to_tmp) {
-    sys = nmalloc(strlen(tempdir) + strlen(nfn) + 1);
-    sprintf(sys, "%s%s", tempdir, nfn);
-    if (file_readable(sys)) {
-      Tcl_AppendResult(irp, "5", NULL);
-      return TCL_OK;
-    }
-    copyfile(argv[1], sys);
-  } else {
-    sys = nmalloc(strlen(argv[1]) + 1);
-    strcpy(sys, argv[1]);
-  }
-  i = raw_dcc_send(sys, argv[2], "*", argv[1]);
-  if (i > 0)
-    wipe_tmp_filename(sys, -1);
+  i = raw_dcc_send(argv[1], argv[2], "*");
   egg_snprintf(s, sizeof s, "%d", i);
   Tcl_AppendResult(irp, s, NULL);
-  nfree(sys);
   return TCL_OK;
 }
 
@@ -81,7 +64,7 @@ static int tcl_getfileq STDVAR
   BADARGS(2, 2, " handle");
 
   for (q = fileq; q; q = q->next) {
-    if (!egg_strcasecmp(q->nick, argv[1])) {
+    if (!strcasecmp(q->nick, argv[1])) {
       s = nrealloc(s, strlen(q->to) + strlen(q->dir) + strlen(q->file) + 4);
       if (q->dir[0] == '*')
         sprintf(s, "%s %s/%s", q->to, &q->dir[1], q->file);
