@@ -504,7 +504,7 @@ static int tcl_control STDVAR
   /* Do not buffer data anymore. All received and stored data is passed
    * over to the dcc functions from now on.  */
   sockoptions(dcc[idx].sock, EGG_OPTION_UNSET, SOCK_BUFFER);
-  strlcpy(dcc[idx].u.script->command, argv[2], 120);
+  strlcpy(dcc[idx].u.script->command, argv[2], sizeof dcc[idx].u.script->command);
   return TCL_OK;
 }
 
@@ -705,11 +705,7 @@ static void dccsocklist(Tcl_Interp *irp, int argc, char *type, int src) {
   char idxstr[10], timestamp[11], other[160];
   char portstring[7]; /* ssl + portmax + NULL */
   long tv;
-#ifdef IPV6
-  char s[INET6_ADDRSTRLEN];
-#else
-  char s[INET_ADDRSTRLEN];
-#endif
+  char s[EGG_INET_ADDRSTRLEN];
   socklen_t namelen;
   struct sockaddr_storage ss;
   Tcl_Obj *masterlist = NULL; /* initialize to NULL to make old gcc versions
@@ -749,11 +745,11 @@ static void dccsocklist(Tcl_Interp *irp, int argc, char *type, int src) {
         getsockname(dcc[i].sock, (struct sockaddr *) &ss, &namelen);
         if (ss.ss_family == AF_INET) {
           struct sockaddr_in *saddr = (struct sockaddr_in *)&ss;
-          inet_ntop(AF_INET, &(saddr->sin_addr), s, INET_ADDRSTRLEN);
+          inet_ntop(AF_INET, &(saddr->sin_addr), s, sizeof s);
 #ifdef IPV6
         } else if (ss.ss_family == AF_INET6) {
           struct sockaddr_in6 *saddr = (struct sockaddr_in6 *)&ss;
-            inet_ntop(AF_INET6, &(saddr->sin6_addr), s, INET6_ADDRSTRLEN);
+            inet_ntop(AF_INET6, &(saddr->sin6_addr), s, sizeof s);
 #endif
         }
         build_sock_list(irp, masterlist, idxstr, dcc[i].nick,
@@ -1032,13 +1028,12 @@ static int tcl_connect STDVAR
 }
 
 static int setlisten(Tcl_Interp *irp, char *ip, char *portp, char *type, char *maskproc, char *flag) {
-  int i, idx = -1, port, realport, found=0, ipv4=1;
-  char s[11], msg[256], newip[INET6_ADDRSTRLEN];
+  int i, idx = -1, port, realport, found=0, ipv4=1, error;
+  char s[11], msg[256], newip[EGG_INET_ADDRSTRLEN];
   struct portmap *pmap = NULL, *pold = NULL;
   sockname_t name;
   struct in_addr ipaddr4;
   struct addrinfo hint, *ipaddr = NULL;
-  int ret;
 #ifdef IPV6
   struct in6_addr ipaddr6;
 #endif
@@ -1060,8 +1055,8 @@ static int setlisten(Tcl_Interp *irp, char *ip, char *portp, char *type, char *m
     strlcpy(newip, ip, sizeof newip);
   }
   /* Return addrinfo struct ipaddr containing family... */
-  ret = getaddrinfo(newip, NULL, &hint, &ipaddr);
-  if (!ret) {
+  error = getaddrinfo(newip, NULL, &hint, &ipaddr);
+  if (!error) {
   /* Load network address to in(6)_addr struct for later byte comparisons */
     if (ipaddr->ai_family == AF_INET) {
       inet_pton(AF_INET, newip, &ipaddr4);
@@ -1072,8 +1067,17 @@ static int setlisten(Tcl_Interp *irp, char *ip, char *portp, char *type, char *m
       ipv4 = 0;
     }
 #endif
+    if (ipaddr) /* The behavior of freeadrinfo(NULL) is left unspecified by RFCs
+                 * 2553 and 3493. Avoid to be compatible with all OSes. */
+      freeaddrinfo(ipaddr);
   }
-  freeaddrinfo(ipaddr);
+  else if (error == EAI_NONAME)
+    /* currently setlisten() handles only ip not hostname */
+    putlog(LOG_MISC, "*",
+           "tcldcc: setlisten(): getaddrinfo(): ip %s not known", newip);
+  else
+    putlog(LOG_MISC, "*", "tcldcc: setlisten(): getaddrinfo(): error = %s",
+           gai_strerror(error));
   port = realport = atoi(portp);
   for (pmap = root; pmap; pold = pmap, pmap = pmap->next) {
     if (pmap->realport == port) {
@@ -1286,7 +1290,7 @@ static int tcl_listen STDVAR
       strlcpy(ip, argv[1], sizeof(ip));
       i++;
     } else {
-      Tcl_AppendResult(irp, "invalid ip address", NULL);
+      Tcl_AppendResult(irp, "invalid IP address argument", NULL);
       return TCL_ERROR;
     }
   }
@@ -1374,12 +1378,12 @@ static int tcl_rehash STDVAR
   BADARGS(1, 1, "");
 
   if (make_userfile) {
-    putlog(LOG_MISC, "*", USERF_NONEEDNEW);
+    putlog(LOG_MISC, "*", "%s", USERF_NONEEDNEW);
     make_userfile = 0;
   }
   write_userfile(-1);
 
-  putlog(LOG_MISC, "*", USERF_REHASHING);
+  putlog(LOG_MISC, "*", "%s", USERF_REHASHING);
   do_restart = -2;
   return TCL_OK;
 }
@@ -1393,11 +1397,11 @@ static int tcl_restart STDVAR
     return TCL_ERROR;
   }
   if (make_userfile) {
-    putlog(LOG_MISC, "*", USERF_NONEEDNEW);
+    putlog(LOG_MISC, "*", "%s", USERF_NONEEDNEW);
     make_userfile = 0;
   }
   write_userfile(-1);
-  putlog(LOG_MISC, "*", MISC_RESTARTING);
+  putlog(LOG_MISC, "*", "%s", MISC_RESTARTING);
   wipe_timers(interp, &utimer);
   wipe_timers(interp, &timer);
   do_restart = -1;
