@@ -57,9 +57,18 @@ static int tcl_finduser STDVAR
 {
   struct userrec *u;
 
-  BADARGS(2, 2, " nick!user@host");
+  BADARGS(2, 3, " ?-account? searchString");
 
-  u = get_user_by_host(argv[1]);
+  if (argc == 3) {
+    if (!strcasecmp(argv[1], "-account")) {
+      u = get_user_by_account(argv[2]);
+    } else {
+      Tcl_AppendResult(irp, "invalid option, must be -account", NULL);
+      return TCL_ERROR;
+    }
+  } else {
+    u = get_user_by_host(argv[1]);
+  }
   Tcl_AppendResult(irp, u ? u->handle : "*", NULL);
   return TCL_OK;
 }
@@ -260,8 +269,9 @@ static int tcl_matchattr STDVAR
       nom = 1;
       if (!plus.global && !plus.udef_global && !plus.chan &&
           !plus.udef_chan && !plus.bot) {
-        Tcl_AppendResult(irp, "Unknown flag specified for matching", NULL);
-        return TCL_ERROR;
+        /* No flags (e.g. "-" or "+" or "-|-" matches anyone */
+        Tcl_AppendResult(irp, "1", NULL);
+        return TCL_OK;
       }
     }
     if (flagrec_eq(&plus, &user)) {
