@@ -135,7 +135,7 @@ int ssl_init()
   /* A TLS/SSL connection established with this method will understand all
      supported protocols (SSLv2, SSLv3, and TLSv1) */
   if (!(ssl_ctx = SSL_CTX_new(SSLv23_method()))) {
-    putlog(LOG_MISC, "*", ERR_error_string(ERR_get_error(), NULL));
+    putlog(LOG_MISC, "*", "%s", ERR_error_string(ERR_get_error(), NULL));
     putlog(LOG_MISC, "*", "ERROR: TLS: unable to create context. Disabling SSL.");
     ERR_free_strings();
     return -1;
@@ -330,12 +330,17 @@ char *ssl_getfp(int sock)
 
   if (!(cert = ssl_getcert(sock)))
     return NULL;
-  if (!X509_digest(cert, EVP_sha1(), md, &i))
+  if (!X509_digest(cert, EVP_sha1(), md, &i)) {
+    X509_free(cert);
     return NULL;
-  if (!(p = OPENSSL_buf2hexstr(md, i)))
+  }
+  if (!(p = OPENSSL_buf2hexstr(md, i))) {
+    X509_free(cert);
     return NULL;
+  }
   strlcpy(fp, p, sizeof fp);
   OPENSSL_free(p);
+  X509_free(cert);
   return fp;
 }
 
@@ -734,7 +739,7 @@ static void ssl_info(const SSL *ssl, int where, int ret)
            "established.");
 
     if ((cert = SSL_get_peer_certificate(ssl))) {
-      ssl_showcert(cert, data->loglevel);
+      ssl_showcert(cert, LOG_DEBUG);
       X509_free(cert);
     }
     else
@@ -743,7 +748,7 @@ static void ssl_info(const SSL *ssl, int where, int ret)
     /* Display cipher information */
     cipher = SSL_get_current_cipher(ssl);
     processed = SSL_CIPHER_get_bits(cipher, &secret);
-    putlog(data->loglevel, "*", "TLS: cipher used: %s %s; %d bits (%d secret)",
+    putlog(LOG_DEBUG, "*", "TLS: cipher used: %s %s; %d bits (%d secret)",
            SSL_CIPHER_get_name(cipher), SSL_get_version(ssl),
            processed, secret);
     /* secret are the actually secret bits. If processed and secret differ,
@@ -1025,6 +1030,7 @@ static int tcl_tlsstatus STDVAR
     Tcl_DStringAppendElement(&ds, "serial");
     Tcl_DStringAppendElement(&ds, p);
     nfree(p);
+    X509_free(cert);
   }
   /* We should always have a cipher, but who knows? */
   cipher = SSL_get_current_cipher(td->socklist[j].ssl);
