@@ -12,7 +12,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2018 Eggheads Development Team
+ * Copyright (C) 1999 - 2022 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -41,13 +41,12 @@
 extern struct dcc_t *dcc;
 extern struct userrec *userlist, *lastuser;
 extern struct chanset_t *chanset;
-extern int dcc_total, noshare;
+extern int dcc_total, noshare, remove_pass;
 extern char botnetnick[];
 extern Tcl_Interp *interp;
 extern time_t now;
 
-char userfile[121] = "";        /* where the user records are stored */
-int ignore_time = 10;           /* how many minutes will ignores last? */
+int ignore_time = 10; /* how many minutes will ignores last? */
 
 /* is this nick!user@host being ignored? */
 int match_ignore(char *uhost)
@@ -60,39 +59,25 @@ int match_ignore(char *uhost)
   return 0;
 }
 
-int equals_ignore(char *uhost)
-{
-  struct igrec *u = global_ign;
-
-  for (; u; u = u->next)
-    if (!rfc_casecmp(u->igmask, uhost)) {
-      if (u->flags & IGREC_PERM)
-        return 2;
-      else
-        return 1;
-    }
-  return 0;
-}
-
 int delignore(char *ign)
 {
   int i, j;
   struct igrec **u;
   struct igrec *t;
-  char temp[256];
+  char temp[UHOSTLEN];
 
   i = 0;
   if (!strchr(ign, '!') && (j = atoi(ign))) {
     for (u = &global_ign, j--; *u && j; u = &((*u)->next), j--);
     if (*u) {
-      strncpyz(temp, (*u)->igmask, sizeof temp);
+      strlcpy(temp, (*u)->igmask, sizeof temp);
       i = 1;
     }
   } else {
     /* find the matching host, if there is one */
     for (u = &global_ign; *u && !i; u = &((*u)->next))
       if (!rfc_casecmp(ign, (*u)->igmask)) {
-        strncpyz(temp, ign, sizeof temp);
+        strlcpy(temp, ign, sizeof temp);
         i = 1;
         break;
       }
@@ -171,7 +156,7 @@ void display_ignore(int idx, int number, struct igrec *ignore)
   if (ignore->flags & IGREC_PERM)
     strcpy(s, "(perm)");
   else {
-    char s1[41];
+    char s1[29];
 
     days(ignore->expire, now, s1);
     sprintf(s, "(expires %s)", s1);
@@ -228,19 +213,17 @@ void check_expired_ignores()
   }
 }
 
-/*        Channel mask loaded from user file. This function is
- *      add(ban|invite|exempt)_fully merged into one. <cybah>
+/* Channel mask loaded from user file. This function is
+ * add(ban|invite|exempt)_fully merged into one. <cybah>
  */
-static void addmask_fully(struct chanset_t *chan, maskrec ** m,
-                          maskrec ** global, char *mask, char *from, char *note,
+static void addmask_fully(maskrec ** m, char *mask, char *from, char *note,
                           time_t expire_time, int flags, time_t added,
                           time_t last)
 {
   maskrec *p = user_malloc(sizeof(maskrec));
-  maskrec **u = (chan) ? m : global;
 
-  p->next = *u;
-  *u = p;
+  p->next = *m;
+  *m = p;
   p->expire = expire_time;
   p->added = added;
   p->lastactive = last;
@@ -285,8 +268,8 @@ static void restore_chanban(struct chanset_t *chan, char *host)
             if (desc) {
               *desc = 0;
               desc++;
-              addmask_fully(chan, &chan->bans, &global_bans, host, user,
-                            desc, atoi(expi), flags, atoi(add), atoi(last));
+              addmask_fully(chan ? &chan->bans : &global_bans, host, user, desc,
+                            atoi(expi), flags, atoi(add), atoi(last));
               return;
             }
           }
@@ -296,7 +279,7 @@ static void restore_chanban(struct chanset_t *chan, char *host)
         if (desc) {
           *desc = 0;
           desc++;
-          addmask_fully(chan, &chan->bans, &global_bans, host, add, desc,
+          addmask_fully(chan ? &chan->bans : &global_bans, host, add, desc,
                         atoi(expi), flags, now, 0);
           return;
         }
@@ -339,7 +322,7 @@ static void restore_chanexempt(struct chanset_t *chan, char *host)
             if (desc) {
               *desc = 0;
               desc++;
-              addmask_fully(chan, &chan->exempts, &global_exempts, host, user,
+              addmask_fully(chan ? &chan->exempts : &global_exempts, host, user,
                             desc, atoi(expi), flags, atoi(add), atoi(last));
               return;
             }
@@ -350,7 +333,7 @@ static void restore_chanexempt(struct chanset_t *chan, char *host)
         if (desc) {
           *desc = 0;
           desc++;
-          addmask_fully(chan, &chan->exempts, &global_exempts, host, add,
+          addmask_fully(chan ? &chan->exempts : &global_exempts, host, add,
                         desc, atoi(expi), flags, now, 0);
           return;
         }
@@ -393,8 +376,9 @@ static void restore_chaninvite(struct chanset_t *chan, char *host)
             if (desc) {
               *desc = 0;
               desc++;
-              addmask_fully(chan, &chan->invites, &global_invites, host, user,
-                            desc, atoi(expi), flags, atoi(add), atoi(last));
+              addmask_fully(chan ? &chan->invites : &global_invites, host, user,
+                            desc, atoi(expi), flags, atoi(add),
+                            atoi(last));
               return;
             }
           }
@@ -404,7 +388,7 @@ static void restore_chaninvite(struct chanset_t *chan, char *host)
         if (desc) {
           *desc = 0;
           desc++;
-          addmask_fully(chan, &chan->invites, &global_invites, host, add,
+          addmask_fully(chan ? &chan->invites : &global_invites, host, add,
                         desc, atoi(expi), flags, now, 0);
           return;
         }
@@ -467,10 +451,10 @@ static void restore_ignore(char *host)
   putlog(LOG_MISC, "*", "*** Malformed ignore line.");
 }
 
-void tell_user(int idx, struct userrec *u, int master)
+static void tell_user(int idx, struct userrec *u)
 {
   char s[81], s1[81], format[81];
-  int n = 0;
+  int n = 0, p = 0;
   time_t now2;
   struct chanuserrec *ch;
   struct user_entry *ue;
@@ -492,14 +476,15 @@ void tell_user(int idx, struct userrec *u, int master)
   else {
     now2 = now - li->laston;
     if (now2 >= 86400)
-      egg_strftime(s1, 11, "%Y-%m-%d", localtime(&li->laston));
+      strftime(s1, 11, "%Y-%m-%d", localtime(&li->laston));
     else
-      egg_strftime(s1, 6, "%H:%M", localtime(&li->laston));
+      strftime(s1, 6, "%H:%M", localtime(&li->laston));
   }
   egg_snprintf(format, sizeof format, "%%-%us %%-5s%%5d %%-15s %%s (%%s)\n",
                HANDLEN);
-  dprintf(idx, format, u->handle,
-          get_user(&USERENTRY_PASS, u) ? "yes" : "no", n, s, s1,
+  if (!u_pass_match(u, "-"))
+    p = 1;
+  dprintf(idx, format, u->handle, p ? "yes" : "no", n, s, s1,
           (li && li->lastonplace) ? li->lastonplace : "nowhere");
   /* channel flags? */
   for (ch = u->chanrec; ch; ch = ch->next) {
@@ -511,9 +496,9 @@ void tell_user(int idx, struct userrec *u, int master)
       else {
         now2 = now - (ch->laston);
         if (now2 >= 86400)
-          egg_strftime(s1, 11, "%Y-%m-%d", localtime(&ch->laston));
+          strftime(s1, 11, "%Y-%m-%d", localtime(&ch->laston));
         else
-          egg_strftime(s1, 6, "%H:%M", localtime(&ch->laston));
+          strftime(s1, 6, "%H:%M", localtime(&ch->laston));
       }
       fr.match = FR_CHAN;
       fr.chan = ch->flags;
@@ -533,7 +518,7 @@ void tell_user(int idx, struct userrec *u, int master)
 }
 
 /* show user by ident */
-void tell_user_ident(int idx, char *id, int master)
+void tell_user_ident(int idx, char *id)
 {
   char format[81];
   struct userrec *u;
@@ -548,14 +533,13 @@ void tell_user_ident(int idx, char *id, int master)
   egg_snprintf(format, sizeof format,
                "%%-%us PASS NOTES FLAGS           LAST\n", HANDLEN);
   dprintf(idx, format, "HANDLE");
-  tell_user(idx, u, master);
+  tell_user(idx, u);
 }
 
 /* match string:
  * wildcard to match nickname or hostmasks
  * +attr to find all with attr */
-void tell_users_match(int idx, char *mtch, int start, int limit,
-                      int master, char *chname)
+void tell_users_match(int idx, char *mtch, int start, int limit, char *chname)
 {
   char format[81];
   struct userrec *u;
@@ -596,7 +580,7 @@ void tell_users_match(int idx, char *mtch, int start, int limit,
         if (nomns || !flagrec_eq(&mns, &user)) {
           cnt++;
           if ((cnt <= limit) && (cnt >= start)) {
-            tell_user(idx, u, master);
+            tell_user(idx, u);
           }
           if (cnt == limit + 1) {
             dprintf(idx, MISC_TRUNCATED, limit);
@@ -606,7 +590,7 @@ void tell_users_match(int idx, char *mtch, int start, int limit,
     } else if (wild_match(mtch, u->handle)) {
       cnt++;
       if ((cnt <= limit) && (cnt >= start)) {
-        tell_user(idx, u, master);
+        tell_user(idx, u);
       }
       if (cnt == limit + 1) {
         dprintf(idx, MISC_TRUNCATED, limit);
@@ -618,7 +602,7 @@ void tell_users_match(int idx, char *mtch, int start, int limit,
           cnt++;
           fnd = 1;
           if ((cnt <= limit) && (cnt >= start)) {
-            tell_user(idx, u, master);
+            tell_user(idx, u);
           }
           if (cnt == limit + 1) {
             dprintf(idx, MISC_TRUNCATED, limit);
@@ -631,6 +615,34 @@ void tell_users_match(int idx, char *mtch, int start, int limit,
   dprintf(idx, MISC_FOUNDMATCH, cnt, cnt == 1 ? "" : MISC_MATCH_PLURAL);
 }
 
+/* if encryption mod and encryption2 mod is loaded and remove-pass is 1 delete
+ * PASS for each user (and bot) that has PASS2 set
+ */
+static void cleanup_pass(void) {
+  struct userrec *u;
+  struct user_entry *e, *p, *p2;
+
+  if (encrypt_pass && encrypt_pass2) {
+    for (u = userlist; u; u = u->next) {
+      p = NULL;
+      p2 = NULL;
+      for (e = u->entries; e; e = e->next) {
+        if (!strcasecmp(e->type->name, "PASS"))
+          p = e;
+        else if (!strcasecmp(e->type->name, "PASS2"))
+          p2 = e;
+      }
+      if (p && p2) {
+        explicit_bzero(p->u.extra, strlen(p->u.extra));
+        nfree(p->u.extra);
+        p->u.extra = NULL;
+        egg_list_delete((struct list_type **) &(u->entries), (struct list_type *) p);
+        nfree(p);
+      }
+    }
+  }
+}
+
 /*
  * tagged lines in the user file:
  * * OLD:
@@ -772,7 +784,7 @@ int readuserfile(char *file, struct userrec **ret)
 
                 cr->next = u->chanrec;
                 u->chanrec = cr;
-                strncpyz(cr->channel, chname, 80);
+                strlcpy(cr->channel, chname, sizeof cr->channel);
                 cr->laston = atoi(st);
                 cr->flags = fr.chan;
                 cr->flags_udef = fr.udef_chan;
@@ -877,7 +889,7 @@ int readuserfile(char *file, struct userrec **ret)
             int ok = 0;
 
             for (ue = u->entries; ue && !ok; ue = ue->next)
-              if (ue->name && !egg_strcasecmp(code + 2, ue->name)) {
+              if (ue->name && !strcasecmp(code + 2, ue->name)) {
                 struct list_type *list;
 
                 list = user_malloc(sizeof(struct list_type));
@@ -936,7 +948,7 @@ int readuserfile(char *file, struct userrec **ret)
             } else {
               fr.match = FR_GLOBAL;
               break_down_flags(attr, &fr, 0);
-              strncpyz(lasthand, code, sizeof lasthand);
+              strlcpy(lasthand, code, sizeof lasthand);
               cst = NULL;
               if (strlen(code) > HANDLEN)
                 code[HANDLEN] = 0;
@@ -949,7 +961,7 @@ int readuserfile(char *file, struct userrec **ret)
 
               u = get_user_by_handle(bu, code);
               for (i = 0; i < dcc_total; i++)
-                if (!egg_strcasecmp(code, dcc[i].nick))
+                if (!strcasecmp(code, dcc[i].nick))
                   dcc[i].user = u;
               u->flags_udef = fr.udef_global;
               /* if s starts with '/' it's got file info */
@@ -967,7 +979,7 @@ int readuserfile(char *file, struct userrec **ret)
   for (u = bu; u; u = u->next) {
     struct user_entry *e;
 
-    if (!(u->flags & USER_BOT) && !egg_strcasecmp(u->handle, botnetnick)) {
+    if (!(u->flags & USER_BOT) && !strcasecmp(u->handle, botnetnick)) {
       putlog(LOG_MISC, "*", "(!) I have a user record, but without +b");
       /* u->flags |= USER_BOT; */
     }
@@ -986,11 +998,13 @@ int readuserfile(char *file, struct userrec **ret)
   }
   noshare = noxtra = 0;
   /* process the user data *now* */
+  if (remove_pass)
+    cleanup_pass();
   return 1;
 }
 
 /* New methodology - cycle through list 3 times
- * 1st time scan for +sh bots and link if none connected
+ * 1st time scan for +(sbcdejnu)h bots and link if none connected
  * 2nd time scan for +h bots
  * 3rd time scan for +a/+h bots */
 void autolink_cycle(char *start)
@@ -1049,7 +1063,7 @@ void autolink_cycle(char *start)
           }
           /* did we make it where we're supposed to start?  yay! */
           if (!ready)
-            if (!egg_strcasecmp(u->handle, start)) {
+            if (!strcasecmp(u->handle, start)) {
               ready = 1;
               autc = NULL;
               /* if starting point is a +h bot, must be in 2nd cycle */
@@ -1067,7 +1081,7 @@ void autolink_cycle(char *start)
           int i;
 
           i = nextbot(u->handle);
-          if ((i >= 0) && !egg_strcasecmp(dcc[i].nick, u->handle)) {
+          if ((i >= 0) && !strcasecmp(dcc[i].nick, u->handle)) {
             char *p = MISC_REJECTED;
 
             /* we're directly connected to the offending bot?! (shudder!) */
@@ -1076,7 +1090,7 @@ void autolink_cycle(char *start)
             dprintf(i, "bye %s\n", BOT_REJECTING);
             killsock(dcc[i].sock);
             lostdcc(i);
-          } else if ((i < 0) && egg_strcasecmp(botnetnick, u->handle)) {
+          } else if ((i < 0) && strcasecmp(botnetnick, u->handle)) {
             /* The bot is not connected, but listed in our tandem list! */
             putlog(LOG_BOTS, "*", "(!) BUG: rejecting not connected bot %s!",
                    u->handle);
@@ -1105,3 +1119,15 @@ void autolink_cycle(char *start)
   if (autc)
     botlink("", -3, autc->handle);      /* try autoconnect */
 }
+
+char *traced_remove_pass(ClientData cd, Tcl_Interp *irp, EGG_CONST char *name1, EGG_CONST char *name2, int flags)
+{
+  const char *value;
+
+  value = Tcl_GetVar2(irp, name1, name2, TCL_GLOBAL_ONLY);
+  if (value[0] == '1' && value[1] == '\0') {
+    cleanup_pass();
+    Tcl_UntraceVar(interp, "remove-pass", TCL_GLOBAL_ONLY|TCL_TRACE_WRITES, traced_remove_pass, NULL);
+  }
+  return NULL;
+}
