@@ -67,7 +67,7 @@ struct user_entry_type {
 #ifndef MAKING_MODS
 extern struct user_entry_type USERENTRY_COMMENT, USERENTRY_LASTON,
   USERENTRY_XTRA, USERENTRY_INFO, USERENTRY_BOTADDR, USERENTRY_HOSTS,
-  USERENTRY_PASS, USERENTRY_PASS2, USERENTRY_BOTFL;
+  USERENTRY_PASS, USERENTRY_PASS2, USERENTRY_BOTFL, USERENTRY_ACCOUNT;
 #ifdef TLS
 extern struct user_entry_type USERENTRY_FPRINT;
 #endif
@@ -183,9 +183,10 @@ extern struct igrec *global_ign;
 struct userrec *adduser();
 struct userrec *get_user_by_handle(struct userrec *, char *);
 struct userrec *get_user_by_host(char *);
+struct userrec *get_user_by_account(char *);
 struct userrec *get_user_by_nick(char *);
-struct userrec *check_chanlist();
-struct userrec *check_chanlist_hand();
+struct userrec *check_chanlist(const char *);
+struct userrec *check_chanlist_hand(const char *);
 
 /* All the default userentry stuff, for code re-use
  */
